"""
$Id: utils.py 55896 2007-12-19 14:36:54Z wichert $
"""
from ZODB.PersistentMapping import PersistentMapping

from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Acquisition import Implicit, aq_inner, aq_parent
import Globals
from zope.i18nmessageid import MessageFactory

from Products.CMFCore.utils import getToolByName
import config

def importModuleFromName(module_name):
    """ import and return a module by its name """
    __traceback_info__ = (module_name, )
    m = __import__(module_name)
    try:
        for sub in module_name.split('.')[1:]:
            m = getattr(m, sub)
    except AttributeError, e:
        raise ImportError(str(e))
    return m

def changeOwnershipOf(object, userid, recursive=0):
    """Changes the ownership of an object. Stolen from Plone and CMFCore, but
    be less restrictive about where the owner is found.
    """
    membership = getToolByName(object, 'portal_membership')

    user = None
    uf = object.acl_users
    while uf is not None:
        user = uf.getUserById(userid)
        if user is not None:
            break
        container = aq_parent(aq_inner(uf))
        parent = aq_parent(aq_inner(container))
        uf = getattr(parent, 'acl_users', None)

    if user is None:
        raise KeyError, "User %s cannot be found." % userid

    object.changeOwnership(user, recursive)

    def fixOwnerRole(object, user_id):
        # Get rid of all other owners
        owners = object.users_with_local_role('Owner')
        for o in owners:
            roles = list(object.get_local_roles_for_userid(o))
            roles.remove('Owner')
            if roles:
                object.manage_setLocalRoles(o, roles)
            else:
                object.manage_delLocalRoles([o])
        # Fix for 1750
        roles = list(object.get_local_roles_for_userid(user_id))
        roles.append('Owner')
        object.manage_setLocalRoles(user_id, roles)

    fixOwnerRole(object, user.getId())
    catalog_tool = getToolByName(object, 'portal_catalog')
    catalog_tool.reindexObject(object)

    if recursive:
        purl = getToolByName(object, 'portal_url')
        _path = purl.getRelativeContentURL(object)
        subobjects = [b.getObject() for b in \
                     catalog_tool(path={'query':_path,'level':1})]
        for obj in subobjects:
            fixOwnerRole(obj, user.getId())
            catalog_tool.reindexObject(obj)

class TransformDataProvider(Implicit):
    """ Base class for data providers """
    security = ClassSecurityInfo()
    security.declareObjectPublic()

    def __init__(self):
        self.config = PersistentMapping()
        self.config_metadata = PersistentMapping()

        self.config.update({'inputs' : {} })
        self.config_metadata.update({
            'inputs' : {
                'key_label' : '',
                'value_label' : '',
                'description' : ''}
            })

    security.declarePublic('setElement')
    def setElement(self, inputs):
        """ inputs - dictionary, but may be extended to new data types"""
        self.config['inputs'].update(inputs)

    def delElement(self, el):
        """ el - dictionary key"""
        del self.config['inputs'][el]

    security.declarePublic('getElements')
    def getElements(self):
        """ Returns mapping """
        return self.config['inputs']

    security.declarePublic('getConfigMetadata')
    def getConfigMetadata(self):
        """ Returns config metadata """
        return self.config_metadata['inputs']

Globals.InitializeClass(TransformDataProvider)

def finalizeSchema(schema):
    # Categorization
    if schema.has_key('subject'):
        schema.changeSchemataForField('subject', 'categorization')
    if schema.has_key('relatedItems'):
        schema.changeSchemataForField('relatedItems', 'categorization')
    if schema.has_key('location'):
        schema.changeSchemataForField('location', 'categorization')
    if schema.has_key('language'):
        schema.changeSchemataForField('language', 'categorization')

    # Dates
    if schema.has_key('effectiveDate'):
        schema.changeSchemataForField('effectiveDate', 'dates')
    if schema.has_key('expirationDate'):
        schema.changeSchemataForField('expirationDate', 'dates')
    if schema.has_key('creation_date'):
        schema.changeSchemataForField('creation_date', 'dates')
    if schema.has_key('modification_date'):
        schema.changeSchemataForField('modification_date', 'dates')

    # Ownership
    if schema.has_key('creators'):
        schema.changeSchemataForField('creators', 'ownership')
    if schema.has_key('contributors'):
        schema.changeSchemataForField('contributors', 'ownership')
    if schema.has_key('rights'):
        schema.changeSchemataForField('rights', 'ownership')

    # Settings
    if schema.has_key('allowDiscussion'):
        schema.changeSchemataForField('allowDiscussion', 'settings')
    if schema.has_key('excludeFromNav'):
        schema.changeSchemataForField('excludeFromNav', 'settings')
    if schema.has_key('nextPreviousEnabled'):
        schema.changeSchemataForField('nextPreviousEnabled', 'settings')

# Use PloneboardMessageFactory for translations in Python
PloneboardMessageFactory = MessageFactory(config.I18N_DOMAIN)
ModuleSecurityInfo('Products.Ploneboard.utils').declarePublic('PloneboardMessageFactory')



## Script (Python) "toPloneboardTime"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=time=None
##title=
##
#given a time string convert it into a DateTime and then format it appropariately
from DateTime import DateTime
ploneboard_time=None
ts = context.translation_service
utranslate = context.utranslate

format = '%Y;%m;%d;%w;%H;%M;%S'

# fallback formats, english
young_format_en = '%A %H:%M' 
old_format_en = '%B %d. %Y'


if not time:
    return 'Unknown date'

try:
    if not isinstance(time, DateTime):
        time = DateTime(str(time))
    (year, month, day, wday, hours, minutes, seconds) = time.strftime(format).split(';')
 
    if time.greaterThan(DateTime()-7):
        ploneboard_time = utranslate("young_date_format: ${wday} ${hours}:${minutes}",
                                     {'wday':utranslate(ts.day_msgid(wday)), 'hours':hours, 'minutes':minutes},
                                      default=time.strftime(young_format_en))
    else:
        ploneboard_time = utranslate("old_date_format: ${year} ${month} ${day} ${hours}:${minutes}",
                                     {'year':year, 'month':utranslate(ts.month_msgid(month)), 
                                      'day':day, 'hours':hours, 'minutes':minutes},
                                      default=time.strftime(old_format_en))

except IndexError:
    pass 

return ploneboard_time



from zope.component import getMultiAdapter
from zope.component import getUtility
from plone.i18n.normalizer.interfaces import IIDNormalizer
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

class SearchView(BrowserView):
    template = ViewPageTemplateFile("search.pt")

    def __init__(self, context, request):
        BrowserView.__init__(self, context, request)
        sp=getToolByName(context, "portal_properties").site_properties
        plone=getMultiAdapter((self.context, self.request), name="plone")

        def crop(text):
            return plone.cropText(text,
                    sp.search_results_description_length, sp.ellipsis)

        self.crop=crop
        self.normalize=getUtility(IIDNormalizer).normalize
        self.icons=plone.icons_visible()
        self.portal=getMultiAdapter((context, request),
                                    name="plone_portal_state").portal_url()+"/"


    def update(self):
        if "q" not in self.request.form:
            self.results=[]
            return

        text=self.request.form["q"]
        for char in [ "(", ")" ]:
            text=text.replace(char, '"%s"' % char)

        ct=getToolByName(self.context, "portal_catalog")
        self.results=ct(
                object_provides="Products.Ploneboard.interfaces.IComment",
               SearchableText=text)


    def info(self, brain):
        """Return display-information for a search result.
        """
        obj=brain.getObject()
        conv=obj.getConversation()
        forum=obj.getForum()
        text=obj.Schema()["text"].get(obj, mimetype="text/plain").strip()

        return dict(
                author = brain.Creator,
                title = brain.Title,
                description = self.crop(text),
                url = brain.getURL()+"/view",
                icon = self.icons and self.portal+brain.getIcon or None,
                forum_url = forum.absolute_url(),
                forum_title = forum.title_or_id(),
                conv_url = conv.absolute_url(),
                conv_title = conv.title_or_id(),
                review_state = self.normalize(brain.review_state),
                portal_type = self.normalize(brain.portal_type),
                relevance = brain.data_record_normalized_score_)


    def board_url(self):
        state=getMultiAdapter((self.context, self.request), name="plone_context_state")
        return state.view_url()


    def __call__(self):
        self.update()
        return self.template()



from zope.interface import implements

from AccessControl import ClassSecurityInfo
from Acquisition import aq_chain, aq_inner
from OFS.Image import File

from Products.CMFCore.utils import getToolByName

from Products.CMFPlone.utils import _createObjectByType
from Products.Archetypes.public import BaseBTreeFolderSchema, Schema
from Products.Archetypes.public import TextField, LinesField, IntegerField
from Products.Archetypes.public import BaseBTreeFolder, registerType
from Products.Archetypes.public import TextAreaWidget, MultiSelectionWidget, IntegerWidget, SelectionWidget
from Products.Archetypes.public import DisplayList

from Products.Ploneboard.config import PROJECTNAME, HAS_SIMPLEATTACHMENT
from Products.Ploneboard.permissions import ViewBoard, ManageForum, AddConversation
from Products.Ploneboard.interfaces import IPloneboard, IForum
from Products.Ploneboard import utils

from Products.CMFPlone.interfaces.NonStructuralFolder \
    import INonStructuralFolder as ZopeTwoINonStructuralFolder
from Products.CMFPlone.interfaces.structure import INonStructuralFolder


schema = BaseBTreeFolderSchema + Schema((
    TextField('description',
              searchable = 1,
              default_content_type = 'text/html',
              default_output_type = 'text/plain',
              widget = TextAreaWidget(
                        description = "Brief description of the forum topic.",
                        description_msgid = "help_description_forum",
                        label = "Description",
                        label_msgid = "label_description_forum",
                        i18n_domain = "ploneboard",
                        rows = 5)),

    LinesField('category',
                 write_permission = ManageForum,
                 vocabulary = 'getCategories',
                 widget = MultiSelectionWidget(
                            description = "Select which category the forum should be listed under. A forum can exist in multiple categories, although using only one category is recommended.",
                            description_msgid = "help_category",
                            condition="object/getCategories",
                            label = "Category",
                            label_msgid = "label_category",
                            i18n_domain = "ploneboard",
                          )),
    IntegerField('maxAttachments',
                write_permission = ManageForum,
                default = 1,
                widget = IntegerWidget(
                         description = "Select the maximum number of attachments per comment.",
                         description_msgid = "help_maxattachments",
                         label = "Maximum number of attachments",
                         label_msgid = "label_maxattachments",
                         i18n_domain = "ploneboard",
                )),
    IntegerField('maxAttachmentSize',
                write_permission = ManageForum,
                vocabulary = 'getAttachmentSizes',
                default = 100,
                widget = SelectionWidget(
                         description = "Select the maximum size for attachments.",
                         description_msgid = "help_maxattachmentsize",
                         label = "Maximum attachment size",
                         label_msgid = "label_maxattachmentsize",
                         i18n_domain = "ploneboard",
                )),
    ))
utils.finalizeSchema(schema)


if not HAS_SIMPLEATTACHMENT:
    schema['maxAttachments'].mode="r"
    schema['maxAttachments'].default=0
    schema['maxAttachments'].widget.visible={'edit' : 'invisible', 'view' : 'invisible' }
    schema['maxAttachmentSize'].widget.visible={'edit' : 'invisible', 'view' : 'invisible' }


class PloneboardForum(BaseBTreeFolder):
    """A Forum contains conversations."""
    implements(IForum, INonStructuralFolder)
    __implements__ = (BaseBTreeFolder.__implements__, ZopeTwoINonStructuralFolder)

    meta_type = 'PloneboardForum'

    schema = schema

    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    def getCatalog(self):
        return getToolByName(self, 'portal_catalog')

    security.declareProtected(ManageForum, 'edit')
    def edit(self, **kwargs):
        """Alias for update()
        """
        self.update(**kwargs)

    security.declarePublic('synContentValues')
    def synContentValues(self):
        return (self.getConversations())

    security.declareProtected(ViewBoard, 'getBoard')
    def getBoard(self):
        """Returns containing or nearest board."""
        # Try containment
        stoptypes = ['Plone Site']
        for obj in aq_chain(aq_inner(self)):
            if hasattr(obj, 'portal_type') and obj.portal_type not in stoptypes:
                if IPloneboard.providedBy(obj):
                    return obj

        return None

    security.declareProtected(AddConversation, 'addConversation')
    def addConversation(self, title, text=None, creator=None, files=None, **kwargs):
        """Adds a new conversation to the forum.

        XXX should be possible to parameterise the exact type that is being
        added.
        """

        id = self.generateId(prefix='')

        conv = _createObjectByType('PloneboardConversation', self, id)

        # XXX: There is some permission problem with AT write_permission
        # and using **kwargs in the _createObjectByType statement.
        conv.setTitle(title)
        if text is not None:
            conv.setDescription(text)

        if creator is not None:
            conv.setCreators([creator])


        if files is not None:
            m = _createObjectByType('PloneboardComment', conv, conv.generateId(prefix=''))

            # XXX: There is some permission problem with AT write_permission
            # and using **kwargs in the _createObjectByType statement.
            m.setTitle(title)
            m.setText(text)

            if creator is not None:
                m.setCreators([creator])

            # Create files in message
            if files:
                for file in files:
                    # Get raw filedata, not persistent object with reference to tempstorage
                    attachment = File(file.getId(), file.title_or_id(), str(file.data), file.getContentType())
                    m.addAttachment(attachment)

            m.reindexObject()

        conv.reindexObject()
        return conv

    security.declareProtected(ViewBoard, 'getConversation')
    def getConversation(self, conversation_id, default=None):
        """Returns the conversation with the given conversation id."""
        #return self._getOb(conversation_id, default)
        catalog = self.getCatalog()
        conversations = catalog(
                object_provides='Products.Ploneboard.interfaces.IConversation',
                getId=conversation_id,
                path='/'.join(self.getPhysicalPath()))
        if conversations:
            return conversations[0].getObject()
        else:
            return None

    security.declareProtected(ManageForum, 'removeConversation')
    def removeConversation(self, conversation_id):
        """Removes a conversation with the given conversation id from the forum."""
        self._delObject(conversation_id)

    security.declareProtected(ViewBoard, 'getConversations')
    def getConversations(self, limit=20, offset=0):
        """Returns conversations."""
        catalog = self.getCatalog()
        return [f.getObject() for f in \
                catalog(object_provides='Products.Ploneboard.interfaces.IConversation',
                        sort_on='modified',
                        sort_order='reverse',
                        sort_limit=(offset+limit),
                        path='/'.join(self.getPhysicalPath()))[offset:offset+limit]]

    security.declareProtected(ViewBoard, 'getNumberOfConversations')
    def getNumberOfConversations(self):
        """Returns the number of conversations in this forum."""
        return len(self.getCatalog()(
            object_provides='Products.Ploneboard.interfaces.IConversation',
            path='/'.join(self.getPhysicalPath())))

    security.declareProtected(ViewBoard, 'getNumberOfComments')
    def getNumberOfComments(self):
        """Returns the number of comments to this forum."""
        return len(self.getCatalog()(
            object_provides='Products.Ploneboard.interfaces.IComment',
            path='/'.join(self.getPhysicalPath())))

    security.declareProtected(ViewBoard, 'getLastConversation')
    def getLastConversation(self):
        """
        Returns the last conversation.
        """
        # XXX Is Created or Modified the most interesting part?
        res = self.getCatalog()(
                object_provides='Products.Ploneboard.interfaces.IConversation',
                sort_on='created', sort_order='reverse', sort_limit=1,
                path='/'.join(self.getPhysicalPath()))
        if res:
            return res[0].getObject()
        else:
            return None

    security.declareProtected(ViewBoard, 'getLastCommentDate')
    def getLastCommentDate(self):
        """
        Returns a DateTime corresponding to the timestamp of the last comment
        for the forum.
        """
        res = self.getCatalog()(
            object_provides='Products.Ploneboard.interfaces.IComment',
            sort_on='created', sort_order='reverse', sort_limit=1,
            path='/'.join(self.getPhysicalPath()))
        if res:
            return res[0].created
        else:
            return None

    security.declareProtected(ViewBoard, 'getLastCommentAuthor')
    def getLastCommentAuthor(self):
        """
        Returns the name of the author of the last comment.
        """
        res = self.getCatalog()(
                object_provides='Products.Ploneboard.interfaces.IComment',
                sort_on='created', sort_order='reverse', sort_limit=1,
                path='/'.join(self.getPhysicalPath()))
        if res:
            return res[0].Creator
        else:
            return None

    # Vocabularies
    security.declareProtected(ViewBoard, 'getCategories')
    def getCategories(self):
        value = []
        board = self.getBoard()
        if board is not None and hasattr(board, 'getCategories'):
            categories = board.getCategories()
            if categories is not None:
                value = [(c,c) for c in categories]
        value.sort()
        return DisplayList(value)

    security.declareProtected(ViewBoard, 'getAttachmentSizes')
    def getAttachmentSizes(self):
        voc = DisplayList()
        voc.add('10', '10 kilobyte')
        voc.add('100', '100 kilobyte')
        voc.add('1000', '1 megabyte')
        voc.add('10000', '10 megabyte')
        voc.add('-1', 'unlimited')

        return voc


    ############################################################################
    # Folder methods, indexes and such

    def __nonzero__(self):
        return 1


registerType(PloneboardForum, PROJECTNAME)



from zope.interface import implements
from AccessControl import ClassSecurityInfo
from Acquisition import aq_inner, aq_chain
from DateTime import DateTime
from OFS.Image import File

from Products.CMFCore.utils import getToolByName

from Products.Archetypes.public import BaseBTreeFolderSchema, Schema, TextField, ReferenceField
from Products.Archetypes.public import BaseBTreeFolder, registerType
from Products.Archetypes.public import RichWidget, ReferenceWidget
from Products.Archetypes.utils import shasattr

from Products.Ploneboard.config import PROJECTNAME, REPLY_RELATIONSHIP

from Products.CMFPlone.utils import _createObjectByType

from Products.Ploneboard.permissions import ViewBoard, AddComment, \
        EditComment, AddAttachment, ManageComment, DeleteComment

from Products.Ploneboard.interfaces import IConversation, IComment
from Products.Ploneboard import utils

from Products.CMFPlone.interfaces.NonStructuralFolder \
    import INonStructuralFolder as ZopeTwoINonStructuralFolder
from Products.CMFPlone.interfaces.structure import INonStructuralFolder

PBCommentBaseBTreeFolderSchema = BaseBTreeFolderSchema.copy()
PBCommentBaseBTreeFolderSchema['title'].read_permission = ViewBoard
PBCommentBaseBTreeFolderSchema['title'].write_permission = EditComment


schema = PBCommentBaseBTreeFolderSchema + Schema((
    TextField('text',
              searchable = 1,
              default_content_type = 'text/html',
              default_output_type = 'text/x-html-safe',
              allowable_content_types=('text/html',
                                       'text/plain'),
              accessor='getText',
              read_permission = ViewBoard,
              write_permission = EditComment,
              widget = RichWidget(description = "Enter comment body.",
                                      description_msgid = "help_text",
                                      label = "Text",
                                      label_msgid = "label_text",
                                      rows = 5,
                                      helper_css = ('ploneboard.css',)
                                      )),
    ReferenceField(
        name='reply_to',
        accessor='inReplyTo', # Suboptimal accessor naming here...
        edit_accessor='inReplyToUID',
        mutator='setInReplyTo',
        relationship=REPLY_RELATIONSHIP,
        widget=ReferenceWidget(visible=False),
        ),
    ))
utils.finalizeSchema(schema)


class PloneboardComment(BaseBTreeFolder):
    """A comment contains regular text body and metadata."""

    # Use RichDocument pattern for attachments
    # Don't inherit from btreefolder...

    implements(IComment, INonStructuralFolder)
    __implements__ = (BaseBTreeFolder.__implements__, ZopeTwoINonStructuralFolder)

    meta_type = 'PloneboardComment'

    schema = schema

    _replies = None       # OIBTree: { id -> 1 }
    _reply_count = None   # A BTrees.Length
    _in_reply_to = None   # Id to comment this is a reply to

    security = ClassSecurityInfo()

    def __init__(self, oid, **kwargs):
        BaseBTreeFolder.__init__(self, oid, **kwargs)
        self.creation_date = DateTime()

    security.declareProtected(EditComment, 'edit')
    def edit(self, **kwargs):
        """Alias for update()
        """
        self.update(**kwargs)

    security.declareProtected(ViewBoard, 'getConversation')
    def getConversation(self):
        """Returns containing conversation."""
        # Try containment
        stoptypes = ['Plone Site']
        for obj in aq_chain(aq_inner(self)):
            if hasattr(obj, 'portal_type') and obj.portal_type not in stoptypes:
                if IConversation.providedBy(obj):
                    return obj
        return None

    security.declareProtected(AddComment, 'addReply')
    def addReply(self,
                 title,
                 text,
                 creator=None,
                 files=None ):
        """Add a reply to this comment."""

        conv = self.getConversation()

        id = conv.generateId(prefix='')
        if not title:
            title = conv.Title()
            if not title.lower().startswith('re:'):
                title = 'Re: ' + title

        m = _createObjectByType(self.portal_type, conv, id)

        # XXX: There is some permission problem with AT write_permission
        # and using **kwargs in the _createObjectByType statement.
        m.setTitle(title)
        m.setText(text)
        m.setInReplyTo(self.UID())

        if creator is not None:
            m.setCreators([creator])

        # Create files in message
        if files:
            for file in files:
                # Get raw filedata, not persistent object with reference to tempstorage
                # file.data might in fact be OFS.Image.Pdata - str will piece it all together
                attachment = File(file.getId(), file.title_or_id(), str(file.data), file.getContentType())
                m.addAttachment(attachment)

        # If this comment is being added by anonymous, make sure that the true
        # owner in zope is the owner of the forum, not the parent comment or
        # conversation. Otherwise, that owner may be able to view or delete
        # the comment.
        membership = getToolByName(self, 'portal_membership')
        if membership.isAnonymousUser():
            forum = self.getConversation().getForum()
            utils.changeOwnershipOf(m, forum.owner_info()['id'], False)

        m.reindexObject()
        conv.reindexObject() # Sets modified
        return m

    security.declareProtected(AddComment, 'deleteReply')
    def deleteReply(self, comment):
        """ Removes comment from the replies index """
        comment.deleteReference(self, REPLY_RELATIONSHIP)

    security.declareProtected(ViewBoard, 'getReplies')
    def getReplies(self):
        """Returns the comments that were replies to this one."""
        # Return backreferences
        return self.getBRefs(REPLY_RELATIONSHIP)

    security.declareProtected(ViewBoard, 'getTitle')
    def getTitle(self):
        """Returns the subject of the comment."""
        return self.Title()

    def childIds(self, level=0):
        """
        Returns list of ids of all child comments, excluding this comment.
        """
        if level == 0:
            result = []
        else:
            result = [self.getId()]
        replies = self.getReplies()
        if replies:
            for msg_object in replies:
                result = result + msg_object.childIds(level+1)
        return result


    security.declareProtected(ManageComment, 'makeBranch')
    def makeBranch(self):
        """"""
        # Contains mappings - old_msg_id -> new_msg_id
        ids = {}

        parent = self.getConversation()
        forum = parent.getForum()
        conv = forum.addConversation(self.getTitle(), self.getText())
        conv.addComment(self.getTitle(), self.getText())
        # here we get id of the first Comment in newly created Conversation
        first_msg_id = conv.objectIds()[0]

        ids.update({self.getId() : first_msg_id})

        objects = map(parent.getComment, self.childIds())
        for obj in objects:
            replyId = obj.inReplyTo().getId()
            comment = conv.getComment(ids.get(replyId))
            msg = comment.addReply(obj.getTitle(), obj.getText())
            ids.update({obj.getId() : msg.getId()})
            # Here we need to set some fields from old objects
            # What else should we update?
            msg.creation_date = obj.creation_date
            msg.setEffectiveDate(obj.EffectiveDate())
            msg.setExpirationDate(obj.ExpirationDate())
            msg.creator = obj.Creator()

        # manually delete all replies
        for msgid in self.childIds():
            parent._delObject(msgid)
        parent._delObject(self.getId()) # delete ourselves and all our descendants
        # if conversation after branching is empty, remove it
        if parent.getNumberOfComments() == 0:
            forum._delObject(parent.getId())
        # we need to reindex stuff in newly created Conversation
        #for o in conv.objectValues():
        #    o.reindexObject()
        return conv

    ###########################
    # Attachment support      #
    ###########################

    def attachmentFilter(self):
        return { 'portal_type' : [
                            'File', 'Image',
                            'ImageAttachment', 'FileAttachment'
                            ],
                }


    security.declareProtected(ViewBoard, 'hasAttachment')
    def hasAttachment(self):
        """Return 0 or 1 if this comment has attachments."""
        return not not self.objectIds(filter=self.attachmentFilter())

    security.declareProtected(AddAttachment, 'validateAddAttachment')
    def validateAddAttachment(self, file):
        def FileSize(file):
            if hasattr(file, 'size'):
                size=file.size
            elif hasattr(file, 'tell'):
                file.seek(0, 2)
                size=file.tell()
                file.seek(0)
            else:
                try:
                    size=len(file)
                except TypeError:
                    size=0

            return size/1024

        if self.getNumberOfAttachments()>=self.getNumberOfAllowedAttachments():
            return False

        maxsize=self.getConversation().getMaxAttachmentSize()
        if maxsize!=-1:
            if FileSize(file)>maxsize:
                return False

        return True


    security.declareProtected(AddAttachment, 'addAttachment')
    def addAttachment(self, file, title=None):
        """ """
        if not self.validateAddAttachment(file):
            raise ValueError, "Attachment could not be added"

        content_type = file.getContentType()
        if content_type.startswith('image/'):
            type_name = 'ImageAttachment'
            mutator = 'setImage'
        else:
            type_name = 'FileAttachment'
            mutator = 'setFile'
        attachment = _createObjectByType(type_name, self, file.getId(),
                title=file.title)
        getattr(attachment, mutator)(file)
        if title is not None:
            attachment.setTitle(title)
        attachment.unmarkCreationFlag()
        if shasattr(attachment, 'at_post_create_script'):
            attachment.at_post_create_script()

    security.declareProtected(AddAttachment, 'removeAttachment')
    def removeAttachment(self, id):
        """ """
        self._delObject(id)

    security.declareProtected(ViewBoard, 'getAttachment')
    def getAttachment(self, id):
        """ """
        return getattr(self, id)

    security.declareProtected(ViewBoard, 'getAttachments')
    def getAttachments(self):
        """ """
        return self.contentValues(filter=self.attachmentFilter())

    security.declareProtected(ViewBoard, 'getNumberOfAttachments')
    def getNumberOfAttachments(self):
        return len(self.contentIds(filter=self.attachmentFilter()))

    security.declareProtected(AddAttachment, 'getNumberOfAllowedAttachments')
    def getNumberOfAllowedAttachments(self):
        """
        Returns number of allowed attachments
        """
        parent = self.getConversation()
        forum = parent.getForum()
        return forum.getMaxAttachments()


    ############################################
    security.declareProtected(ViewBoard, 'getText')
    def getText(self, mimetype=None, **kwargs):
        """  """
        # Maybe we need to set caching for transform?

        orig = self.getRawText()

        pb_tool = getToolByName(self, 'portal_ploneboard')
        return pb_tool.performCommentTransform(orig, context=self)

    security.declareProtected(ViewBoard, 'Description')
    def Description(self, **kwargs):
        """We have to override Description here to handle arbitrary
        arguments since PortalFolder defines it."""
        if kwargs.get('mimetype', None) is None:
            kwargs['mimetype'] = 'text/plain'
        return self.getField('text').get(self, **kwargs)

    security.declareProtected(DeleteComment, "delete")
    def delete(self):
        """Delete this comment and make sure all comment replies to this
        comment are also cleaned up.
        """

        parent_comment = self.inReplyTo()
        for reply in self.getReplies():
            reply.setInReplyTo(parent_comment)
            reply.reindexObject()

        conversation = self.getConversation()
        conversation._delObject(self.getId())
        conversation.reindexObject()

    def __nonzero__(self):
        return 1

    def __str__(self):
        return "<PloneboardComment: title=%r;>" % self.Title()
    __repr__  = __str__


    security.declareProtected(DeleteComment, "object_delete")
    def object_delete(self):
        """Delete the comment the 'proper' way.
        """

        return self.restrictedTraverse('@@delete_view')()

registerType(PloneboardComment, PROJECTNAME)



#
# Conversation tests
#

import unittest
from zope.interface.verify import verifyClass, verifyObject
import PloneboardTestCase

from Products.CMFPlone.utils import _createObjectByType
from Products.Ploneboard.interfaces import IConversation
from Products.Ploneboard.content.PloneboardConversation import PloneboardConversation

from Products.Ploneboard.tests.utils import addMember

class TestPloneboardConversation(PloneboardTestCase.PloneboardTestCase):

    def afterSetUp(self):
        self.board = _createObjectByType('Ploneboard', self.folder, 'board')
        self.forum = _createObjectByType('PloneboardForum', self.board, 'forum')
        self.conv = self.forum.addConversation('subject', 'body')
        self.comment = self.conv.addComment('subject', 'body')

    def testInterfaceVerification(self):
        self.failUnless(verifyClass(IConversation, PloneboardConversation))

    def testInterfaceConformance(self):
        self.failUnless(IConversation.providedBy(self.conv))
        self.failUnless(verifyObject(IConversation, self.conv))

    def testGetForum(self):
        self.failUnlessEqual(self.forum, self.conv.getForum())

    def testAddComment(self):
        msg = self.conv.addComment('msg_title', 'msg_text')
        self.assertEqual(msg.getTitle(), 'msg_title')
        self.assertEqual(msg.getText(), 'msg_text')

    def testGetLastComment(self):
        msg = self.conv.addComment('last_comment_title', 'last_comment_text')
        self.assertEqual(self.conv.getLastComment().getTitle(), msg.getTitle())
        msg2 = self.conv.addComment('last_comment_title2', 'last_comment_text')
        self.assertEqual(self.conv.getLastComment().getTitle(), msg2.getTitle())

    def testGetComment(self):
        conv = self.conv
        self.failUnlessEqual(self.comment, conv.getComment(self.comment.getId()))

    def testGetComments(self):
        conv = self.conv
        comment2 = conv.addComment('subject2', 'body2')
        self.failUnlessEqual(conv.getComments(), [self.comment, comment2]) 

    def testGetCommentsSlicing(self):
        conv = self.conv
        comment2 = conv.addComment('subject2', 'body2')
        self.failUnlessEqual(conv.getComments(limit=1, offset=0), [self.comment]) 
        self.failUnlessEqual(conv.getComments(limit=1, offset=1), [comment2]) 

    def testGetNumberOfComments(self):
        conv = self.conv
        self.failUnlessEqual(conv.getNumberOfComments(), 1)
        conv.addComment('followup', 'text')
        self.failUnlessEqual(conv.getNumberOfComments(), 2)
        # Check to make sure it doesn't count comments elsewhere
        conv2 = self.forum.addConversation('subject', 'body')
        self.failUnlessEqual(conv.getNumberOfComments(), 2)

    def testGetLastCommentDate(self):
        conv = self.conv
        self.failUnlessEqual(self.comment.created(), conv.getLastCommentDate())
        comment = conv.addComment('followup', 'text')
        self.failUnlessEqual(comment.created(), conv.getLastCommentDate())

    def testGetRootComments(self):
        conv = self.conv
        threaded = conv.getRootComments()
        self.failUnlessEqual(len(threaded), 1)

        conv.addComment("base2", "base two")
        threaded = conv.getRootComments()
        self.failUnlessEqual(len(threaded), 2)

        reply = self.comment.addReply('anotherfollowup', 'moretext')
        threaded = conv.getRootComments()
        self.failUnlessEqual(len(threaded), 2)
        self.failUnlessEqual(len(self.comment.getReplies()), 1)

    def testGetFirstComment(self):
        conv = self.conv
        first = conv.getFirstComment()
        self.failUnless(first)
        self.failUnlessEqual(first, conv.objectValues()[0])
        conv.addComment('followup', 'text')
        self.failUnlessEqual(first, conv.getFirstComment())

    def testModificationDate(self):
        conv = self.conv
        modified1 = conv.modified()
        from time import sleep
        sleep(0.1) # To make sure modified is different
        conv.addComment('followup', 'text')
        modified2 = conv.modified()
        self.failIfEqual(modified1, modified2)
        conv.objectValues()[0].addReply('followup', 'text')
        sleep(0.1) # To make sure modified is different
        modified3 = conv.modified()
        self.failIfEqual(modified1, modified3)
        self.failIfEqual(modified2, modified3)

    def XXXtest_delObject(self):
        forum = self.forum
        conv = forum.addConversation('subject', 'body')
        msg = conv.addComment('msg_title', 'msg_text')
        #msg = conv.objectValues()[0]
        r = msg.addReply('reply_subject', 'reply_body')
        r1 = msg.addReply('reply_subject1', 'reply_body1')
        r2 = msg.addReply('reply_subject2', 'reply_body2')
        r2.addReply('rs', 'rb').addReply('rs1', 'rb1').addReply('rs2', 'rb2')
        self.assertEqual(conv.getNumberOfComments(), 7)
        conv._delObject(r2.getId(), recursive=1)
        self.assertEqual(conv.getNumberOfComments(), 3)
        # check if we delete conversation so we delete root comment
        self.assertEqual(forum.getNumberOfConversations(), 1)
        conv._delObject(msg.getId(), recursive=1)
        self.assertEqual(forum.getNumberOfConversations(), 0)
        # check if Comment was uncataloged
        self.failIf(r.getId() in [v.id for v in self.catalog(meta_type='PloneboardComment', id=r.getId())])
        self.failIf(msg.getId() in [v.id for v in self.catalog(meta_type='PloneboardComment', id=msg.getId())])
        
        # Checking non recursive delete
        conv = forum.addConversation('subject', 'body')
        msg = conv.objectValues()[0]
        r = msg.addReply('reply_subject', 'reply_body')
        self.assertEqual(conv.getNumberOfComments(), 2)
        self.failUnless(r.getId() in [v.getId for v in self.catalog(meta_type='PloneboardComment', id=r.getId())])
        conv._delObject(msg.getId())
        self.assertEqual(conv.getNumberOfComments(), 1)
        self.failUnless(r.getId() in [v.getId for v in self.catalog(meta_type='PloneboardComment', id=r.getId())])
        
    def testNewConversationIsVisibleToAnonymous(self):
        conv = self.forum.addConversation('subject2', 'body2')
        conv.addComment("comment", "comment")
        id = conv.getId()
        self.logout()
        convs = self.forum.getConversations()
#        self.failUnless(id in [x.getId() for x in convs])
        comments = conv.getComments()
        self.assertEqual(len(comments), 1)
        
    def testAddCommentAsAnonymousTakesOwnerOfForumAndCreatorAnonymous(self):
        conv = self.conv
        self.logout()
        reply = conv.addComment('reply1', 'body1', creator='Anonymous')
        self.assertEqual(conv.getForum().owner_info()['id'], reply.owner_info()['id'])
        self.assertEqual(reply.Creator(), 'Anonymous')
    
    def testAddCommentAsNotAnonymousLeavesOwnershipAlone(self):
        conv = self.conv
        addMember(self, 'member2')
        self.login('member2')
        self.assertNotEqual(conv.getForum().owner_info()['id'], 'member2')
        reply = conv.addComment('reply1', 'body1')
        self.assertEqual(reply.owner_info()['id'], 'member2')
        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestPloneboardConversation))
    return suite



#
# Comment tests
#

import unittest
from zope.interface.verify import verifyClass

import PloneboardTestCase
from Products.Ploneboard.interfaces import IComment
from Products.Ploneboard.content.PloneboardComment import PloneboardComment
from Products.Ploneboard.config import HAS_SIMPLEATTACHMENT

from OFS.Image import File
from Products.CMFPlone.utils import _createObjectByType

from Products.Ploneboard.tests.utils import addMember

class TestPloneboardComment(PloneboardTestCase.PloneboardTestCase):

    def afterSetUp(self):
        self.board = _createObjectByType('Ploneboard', self.folder, 'board')
        self.forum = _createObjectByType('PloneboardForum', self.board, 'forum')
        self.conv = self.forum.addConversation('conv1', 'conv1 body')
        self.comment = self.conv.addComment("comment1", "comment1 body")

    def testInterfaceVerification(self):
        self.failUnless(verifyClass(IComment, PloneboardComment))

    def testGetConversation(self):
        self.failUnlessEqual(self.comment.getConversation(), self.conv)

    def testAddReply(self):
        conv = self.conv
        reply = self.comment.addReply('reply1', 'body1')
        self.failUnless(reply in conv.objectValues())

    def testAddReplyAsAnonymousTakesOwnerOfForumAndCreatorAnonymous(self):
        conv = self.conv
        self.logout()
        reply = self.comment.addReply('reply1', 'body1', creator='Anonymous')
        self.assertEqual(conv.getForum().owner_info()['id'], reply.owner_info()['id'])
        self.assertEqual(reply.Creator(), 'Anonymous')

    def testAddReplyAsNotAnonymousLeavesOwnershipAlone(self):
        conv = self.conv
        addMember(self, 'member2')
        self.login('member2')
        self.assertNotEqual(conv.getForum().owner_info()['id'], 'member2')
        reply = self.comment.addReply('reply1', 'body1')
        self.assertEqual(reply.owner_info()['id'], 'member2')

    def testAddReplyAddsRe(self):
        conv = self.conv
        reply = self.comment.addReply('', 'body1')
        self.assertEqual(reply.Title(), 'Re: ' + conv.Title())

    def testAddReplyAddsReOnlyOnce(self):
        conv = self.conv
        reply = self.comment.addReply('', 'body1')
        reply2 = reply.addReply('', 'body2')
        self.assertEqual(reply2.Title(), 'Re: ' + conv.Title())

    def testAddReplyOnlyAddsReIfNotSet(self):
        conv = self.conv
        reply = self.comment.addReply('reply1', 'body1')
        self.assertEqual(reply.Title(), 'reply1')

    def testInReplyTo(self):
        reply = self.comment.addReply('reply1', 'body1')
        self.failUnlessEqual(self.comment, reply.inReplyTo())

    def testGetReplies(self):
        reply = self.comment.addReply('reply1', 'body1')
        reply2 = self.comment.addReply('reply2', 'body2')
        self.failUnlessEqual(len(self.comment.getReplies()), 2)
        self.failUnless(reply in self.comment.getReplies())
        self.failUnless(reply2 in self.comment.getReplies())

    def testGetTitle(self):
        self.failUnlessEqual(self.comment.getTitle(), 'comment1')

    def testGetText(self):
        self.failUnlessEqual(self.comment.getText(), 'comment1 body')

    def testSetInReplyTo(self):
        forum = self.forum
        conv = forum.addConversation('subject', 'body')
        msg = conv.addComment('msg_subject', 'msg_body')
        msg1 = conv.addComment('msg_subject1', 'msg_body1')
        msg1.setInReplyTo(msg)
        self.assertEqual(msg.getId(), msg1.inReplyTo().getId())

    def testDeleteReply(self):
        conv = self.conv

        m = conv.objectValues()[0]
        self.assertEqual(conv.getNumberOfComments(), 1)
        r = m.addReply('reply1', 'body1')
        self.assertEqual(conv.getNumberOfComments(), 2)

        m.deleteReply(r)
        self.assertEqual(len(m.getReplies()), 0)

    def testMakeBranch(self):
        forum = self.forum
        conv = self.conv

        comment = conv.objectValues()[0]
        reply = comment.addReply('reply1', 'body1')
        reply1 = reply.addReply('reply2', 'body2')
        self.assertEqual(conv.getNumberOfComments(), 3)
        self.assertEqual(forum.getNumberOfConversations(), 1)

        branch = reply.makeBranch()

        self.assertEqual(conv.getNumberOfComments(), 1)
        self.assertEqual(forum.getNumberOfConversations(), 2)

        self.failIfEqual(branch, conv)
        self.assertEqual(branch.getNumberOfComments(), 2)

    def testChildIds(self):
        conv = self.conv
        r = self.comment.addReply('reply_subject', 'reply_body')
        r1 = self.comment.addReply('reply_subject1', 'reply_body1')
        r2 = self.comment.addReply('reply_subject2', 'reply_body2')
        r2.addReply('rs', 'rb').addReply('rs1', 'rb1').addReply('rs2', 'rb2')
        self.assertEqual(len(self.comment.childIds()), 6)

    def testTransforms(self):
        conv = self.conv
        text = 'Smiley :)'
        self.comment.setText(text)
        self.failUnless(self.comment.getText())
        self.failIfEqual(self.comment.getText(), text)
        self.failUnlessEqual(self.portal.portal_ploneboard.performCommentTransform(text), self.comment.getText())

    def XXXtestDeleting(self):
        pass

    def testNewCommentIsVisibleToAnonymous(self):
        comment = self.conv.addComment('subject2', 'body2')
        id = comment.getId()
        self.logout()
        comments = self.conv.getComments()
        self.failUnless(id in [x.getId() for x in comments])




class TestPloneboardCommentAttachmentSupport(PloneboardTestCase.PloneboardTestCase):

    def afterSetUp(self):
        self.board = _createObjectByType('Ploneboard', self.folder, 'board')
        self.forum = _createObjectByType('PloneboardForum', self.board, 'forum')
        self.conv = self.forum.addConversation('conv1', 'conv1 body')
        self.comment = self.conv.addComment("comment1", "comment1 body")

    def testAddAttachment(self):
        conv = self.conv

        self.assertEqual(self.comment.getNumberOfAttachments(), 0)
        file = File('testfile', 'testtitle', 'asdf')
        self.comment.addAttachment(file=file, title='comment')
        self.assertEqual(self.comment.getNumberOfAttachments(), 1)
        self.failUnless(self.comment.getAttachment('testfile'))

    def testHasAttachment(self):
        pass

    def testRemoveAttachment(self):
        conv = self.conv

        file = File('testfile', 'testtitle', 'asdf')
        self.comment.addAttachment(file=file, title='comment')
        self.assertEqual(self.comment.getNumberOfAttachments(), 1)
        self.comment.removeAttachment('testfile')
        self.assertEqual(self.comment.getNumberOfAttachments(), 0)


    def testAttachmentRestrictionChanging(self):
        conv = self.conv
        self.forum.setMaxAttachments(10)
        self.failUnlessEqual(self.comment.getNumberOfAllowedAttachments(), 10)
        self.forum.setMaxAttachments(1)
        self.failUnlessEqual(self.comment.getNumberOfAllowedAttachments(), 1)


    def tryAttachmentSizeRestrictions(self, msg):
        self.forum.setMaxAttachments(10)

        self.forum.setMaxAttachmentSize(1)
        file = File('testfile', 'testtitle', 'X')
        msg.addAttachment(file=file, title='comment')
        msg.removeAttachment('testfile')

        file = File('testfile', 'testtitle', 'X'*2048)
        try:
            msg.addAttachment(file=file, title='comment')
        except ValueError:
            pass
        else:
            self.fail("Can add too many attachments")

        self.forum.setMaxAttachmentSize(2)
        msg.addAttachment(file=file, title='comment')


    def testAttachmentSizeRestriction(self):
        conv = self.conv
        self.tryAttachmentSizeRestrictions(self.comment)

    def testAttachmentSizeRestrictionsOnChild(self):
        conv = self.conv
        reply = self.comment.addReply('reply1', 'body1')
        self.tryAttachmentSizeRestrictions(reply)


    def testAttachmentNumberRestriction(self):
        conv = self.conv
        self.forum.setMaxAttachments(1)
        file = File('testfile', 'testtitle', 'asdf')
        self.comment.addAttachment(file=file, title='comment')
        file = File('testfile2', 'testtitle2', 'asdf')
        try:
            self.comment.addAttachment(file=file, title='another comment')
        except ValueError:
            pass
        else:
            self.fail("Can add too many attachments")


    def testGetAttachments(self):
        conv = self.conv
        self.forum.setMaxAttachments(5)

        file = File('testfile', 'testtitle', 'asdf')
        self.comment.addAttachment(file=file, title='comment')
        file1 = File('testfile1', 'testtitle1', 'asdf')
        self.comment.addAttachment(file=file1, title='comment1')
        self.assertEqual(len(self.comment.getAttachments()), 2)
        self.failUnless('comment' in [v.Title() for v in self.comment.getAttachments()])
        self.failUnless('comment1' in [v.Title() for v in self.comment.getAttachments()])

    def testDeleteing(self):
        """Test deleting a comment.
        """

        # Was going to use doctests for this until I realized that
        # PloneTestCase has no doctest capability :(
        # - Rocky

        # Actually - it does!
        # see http://plone.org/documentation/tutorial/testing :)
        # - Martin

        # Now lets start adding some comments:
        first_comment = self.conv.getFirstComment()
        c1 = first_comment.addReply('foo1', 'bar1')
        c2 = first_comment.addReply('foo2', 'bar2')
        c21 = c2.addReply('foo3', 'bar3')

        # Make sure the first comment has exactly two replies:

        self.assert_(first_comment.getReplies(), [c1, c2])

        # Now lets try deleting the first reply to the main comment:

        c1.delete()
        self.assert_(first_comment.getReplies(), [c2])

        # Ok, so lets try deleting a comment that has replies to it:

        c2.delete()

        # Now even though we deleted the last remaining reply to
        # first_comment, we should still have another reply because
        # deleting a reply that had a child will make that child seem
        # as though it is a reply to its parent's parent.

        self.assert_(first_comment.getReplies(), [c21])

        # lets add a comment to c21

        c211 = c21.addReply('foo4', 'bar4')

        # Once the only root comment is deleted that means the conversation's
        # sole root comment should become c211

        c21.delete()
        self.assert_(self.conv.getRootComments(), [c211])

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestPloneboardComment))
    if HAS_SIMPLEATTACHMENT:
        suite.addTest(unittest.makeSuite(TestPloneboardCommentAttachmentSupport))

    return suite



#
# Ploneboard tests
#

import unittest
from zope.interface.verify import verifyClass, verifyObject
from Products.Ploneboard.tests import PloneboardTestCase
from Products.Ploneboard.interfaces import IPloneboard, IForum
from Products.Ploneboard.content.Ploneboard import Ploneboard

# Catch errors in Install
from Products.Ploneboard.Extensions import Install

from Products.CMFPlone.utils import _createObjectByType


class TestPloneboardBasics(PloneboardTestCase.PloneboardTestCase):
    """Test the basics, like creation"""

    def testPloneboardCreation(self):
        """Try creating and deleting a board"""
        board_id = 'board'
        board = _createObjectByType('Ploneboard', self.portal, board_id)
        self.failIfEqual(board, None)
        self.failUnless(board_id in self.portal.objectIds())
        self.portal._delObject(board_id)
        self.failIf(board_id in self.portal.objectIds())


class TestPloneboardInterface(PloneboardTestCase.PloneboardTestCase):
    """Test if it fulfills the interface"""

    def afterSetUp(self):
        self.board = _createObjectByType('Ploneboard', self.folder, 'board')

    def testInterfaceVerification(self):
        self.failUnless(verifyClass(IPloneboard, Ploneboard))

    def testInterfaceConformance(self):
        self.failUnless(IPloneboard.providedBy(self.board))
        self.failUnless(verifyObject(IPloneboard, self.board))        

    def testAddForum(self):
        """Create new folder in home directory & check its basic 
        properties and behaviour"""
        board = self.board
        forum_id = 'forum'
        board.addForum(forum_id, 'title', 'description')
        self.failUnless(forum_id in board.objectIds())
        forum = getattr(board, forum_id)
        self.failUnless(IForum.providedBy(forum))

    def testRemoveForum(self):
        board = self.board
        forum_id = 'forum'
        board.addForum(forum_id, 'title', 'description')
        board.removeForum(forum_id)
        self.failIf(forum_id in board.objectIds())

    def testGetForum(self):
        board = self.board
        forum_id = 'forum'
        board.addForum(forum_id, 'title', 'description')
        forum = board.getForum(forum_id)
        self.failUnless(IForum.providedBy(forum))

    def testGetForumIds(self):
        board = self.board
        forum_ids = ['forum1', 'forum2']
        for forum_id in forum_ids:
            board.addForum(forum_id, 'title', 'description')
        self.failUnlessEqual(forum_ids, board.getForumIds())

    def testGetForums(self):
        board = self.board
        forum_ids = ['forum1', 'forum2']
        for forum_id in forum_ids:
            board.addForum(forum_id, 'title', 'description')
        self.failUnlessEqual(
                set([board.getForum(forum_id) for forum_id in forum_ids]),
                set(board.getForums()))

    def testSearchComments(self):
        pass


class TestPloneboardWithoutContainment(PloneboardTestCase.PloneboardTestCase):
    """Test a single board used more as a topic, with forums in folders"""

    def afterSetUp(self):
        self.board = _createObjectByType('Ploneboard', self.folder, 'board')

    def testGetForumsOutsideBoard(self):
        forum1 = _createObjectByType('PloneboardForum', self.folder, 'forum1')
        forum2 = _createObjectByType('PloneboardForum', self.folder, 'forum2')
        forums = self.board.getForums(sitewide=True)
        self.failUnless(forum1 in forums)
        self.failUnless(forum2 in forums)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestPloneboardBasics))
    suite.addTest(unittest.makeSuite(TestPloneboardInterface))
    suite.addTest(unittest.makeSuite(TestPloneboardWithoutContainment))
    return suite



#
# Comment tests
#

import unittest
import PloneboardTestCase
from Products.Ploneboard.transforms.url_to_hyperlink import URLToHyperlink

class MockTransformData:
    def setData(self, value):
        self.data=value


class TestUrlTransform(PloneboardTestCase.PloneboardTestCase):
    def runTest(self, testdata):
        transform = URLToHyperlink()
        data=MockTransformData()
        for (input, answer) in testdata:
            transform.convert(input, data)
            self.failUnlessEqual(data.data, answer)

    def testPlainText(self):
        testdata = [
                ("just a simple string",  "just a simple string"),
                ("htXtp:bla invalid scheme", "htXtp:bla invalid scheme"),
                ]
        self.runTest(testdata)

    def testPlainUrls(self):
        testdata = [
                ("http://simple.url/", '<a href="http://simple.url/">http://simple.url/</a>'),
                # XXX are URI schemes really case insensitive?
                ("HTtp://simple.url/", '<a href="HTtp://simple.url/">HTtp://simple.url/</a>'),
                ("https://simple.url/", '<a href="https://simple.url/">https://simple.url/</a>'),
                ("telnet://simple.url/", '<a href="telnet://simple.url/">telnet://simple.url/</a>'),
                ]
        self.runTest(testdata)

    def testUrlElements(self):
        testdata = [
                ("<telnet://simple.url/>", '<telnet://simple.url/>'),
                ("< telnet://simple.url/>", '< telnet://simple.url/>'),
                ("<telnet://simple.url />", '<telnet://simple.url />'),
                ("http://change.this/ <telnet://simple.url/>", '<a href="http://change.this/">http://change.this/</a> <telnet://simple.url/>'),
                ]
        self.runTest(testdata)


    def testEmail(self):
        testdata = [
                ("test@example.com", '<a href="mailto:test@example.com">test@example.com</a>'),
                ("<test@example.com>", "<test@example.com>"),
                ]
        self.runTest(testdata)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestUrlTransform))
    
    return suite




"""
$Id: config.py 55896 2007-12-19 14:36:54Z wichert $
"""
PROJECTNAME = "Ploneboard"
SKINS_DIR = 'skins'
I18N_DOMAIN = PROJECTNAME.lower()

# Transform config
EMOTICON_TRANSFORM_ID = 'text_to_emoticons'
EMOTICON_TRANSFORM_MODULE = 'Products.Ploneboard.transforms.text_to_emoticons'
URL_TRANSFORM_MODULE = 'Products.Ploneboard.transforms.url_to_hyperlink'
SAFE_HTML_TRANSFORM_MODULE = 'Products.PortalTransforms.transforms.safe_html'

PLONEBOARD_TRANSFORMSCHAIN_ID = 'ploneboard_chain'
PLONEBOARD_TOOL = 'portal_ploneboard'

REPLY_RELATIONSHIP = 'ploneboard_reply_to'
# This should be configurable ttw
NUMBER_OF_ATTACHMENTS = 5

GLOBALS = globals()

try:
    from Products import SimpleAttachment as SA
    HAS_SIMPLEATTACHMENT = True
    del SA
except ImportError:
    HAS_SIMPLEATTACHMENT = False




from Acquisition import aq_inner, aq_parent
from Products.Ploneboard.interfaces import IConversation, IComment


def autopublish_script(self, sci):
    """Publish the conversation along with the comment"""
    object = sci.object

    wftool = sci.getPortal().portal_workflow

    # Try to make sure that conversation and contained messages are in sync
    if IComment.providedBy(object):
        parent = aq_parent(aq_inner(object))
        if IConversation.providedBy(parent):
            try:
                if wftool.getInfoFor(parent,'review_state', None) in (sci.old_state.getId(), 'pending'):
                    wftool.doActionFor(parent, 'publish')
            except:
                pass

def publish_script(self, sci):
    """Publish the conversation along with comment"""
    object = sci.object

    wftool = sci.getPortal().portal_workflow

    if IComment.providedBy(object):
        parent = aq_parent(aq_inner(object))
        if IConversation.providedBy(parent):
            try:
                if wftool.getInfoFor(parent,'review_state', None) in (sci.old_state.getId(), 'pending'):
                    wftool.doActionFor(parent, 'publish')
            except:
                pass

def reject_script(self, sci):
    """Reject conversation along with comment"""
    # Dispatch to more easily customizable methods
    object = sci.object
    # We don't have notifyPublished method anymore
    #object.notifyRetracted()

    wftool = sci.getPortal().portal_workflow

    # Try to make sure that conversation and contained messages are in sync
    if IComment.providedBy(object):
        parent = aq_parent(aq_inner(object))
        if IConversation.providedBy(parent):
            try:
                 if wftool.getInfoFor(parent,'review_state', None) in (sci.old_state.getId(), 'pending'):
                    wftool.doActionFor(parent, 'reject')
            except:
                pass



from Products.CMFCore.utils import getToolByName
from Products.Ploneboard.config import EMOTICON_TRANSFORM_MODULE
from Products.Ploneboard.config import URL_TRANSFORM_MODULE
from Products.Ploneboard.config import SAFE_HTML_TRANSFORM_MODULE

def setupVarious(context):
    site=context.getSite()

    addTransforms(site)


def addTransforms(site):
    pb_tool = getToolByName(site, 'portal_ploneboard')
    pb_tool.registerTransform('text_to_emoticons', EMOTICON_TRANSFORM_MODULE, 'Graphical smilies')
    pb_tool.registerTransform('url_to_hyperlink', URL_TRANSFORM_MODULE, 'Clickable links')
    pb_tool.registerTransform('safe_html', SAFE_HTML_TRANSFORM_MODULE, 'Remove dangerous HTML')




"""
$Id: PloneboardTool.py 55896 2007-12-19 14:36:54Z wichert $
"""
import Globals
from AccessControl import ClassSecurityInfo
from OFS.Image import File
from OFS.Folder import Folder
from ZPublisher.HTTPRequest import FileUpload
from ZODB.PersistentMapping import PersistentMapping

from zope.interface import implements

from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import registerToolInterface
from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.ActionProviderBase import ActionProviderBase

from Products.Ploneboard.config import PLONEBOARD_TOOL
from Products.Ploneboard.interfaces import IPloneboardTool

class PloneboardTool(UniqueObject, Folder, ActionProviderBase):
    implements(IPloneboardTool)

    id = PLONEBOARD_TOOL
    meta_type = 'Ploneboard Tool'

    security = ClassSecurityInfo()
    def __init__(self):
        self.transforms = PersistentMapping()

    security.declarePrivate('registerTransform')
    def registerTransform(self, name, module, friendlyName=None):
        tr_tool = getToolByName(self, 'portal_transforms')
        if name not in tr_tool.objectIds():
            tr_tool.manage_addTransform(name, module)
            wasAdded = True
        else:
            wasAdded = False

        if not friendlyName:
            friendlyName = name

        if name not in self.transforms:
            self.transforms[name] = {'enabled' : True,
                                     'friendlyName' : friendlyName,
                                     'wasAdded' : wasAdded
                                     }

    security.declarePrivate('unregisterTransform')
    def unregisterTransform(self, name):
        tr_tool = getToolByName(self, 'portal_transforms')
        if self.transforms[name]['wasAdded']:
            try:
                tr_tool._delObject(name)
            except AttributeError, e:
                pass
        del self.transforms[name]

    security.declareProtected(ManagePortal, 'enableTransform')
    def enableTransform(self, name, enabled=True):
        """Change the activity status for a transform."""
        self.transforms[name]['enabled'] = enabled

    security.declarePrivate('unregisterAllTransforms')
    def unregisterAllTransforms(self):
        tr_tool = getToolByName(self, 'portal_transforms')
        for transform_name in self.getTransforms():
            if not self.transforms[transform_name] or \
                    self.transforms[transform_name].get('wasAdded', False):
                try:
                    tr_tool._delObject(transform_name)
                except AttributeError:
                    pass
        self.transforms.clear()

    security.declareProtected(ManagePortal, 'getTransforms')
    def getTransforms(self):
        """Returns list of transform names."""
        return self.transforms.keys()

    security.declareProtected(ManagePortal, 'getTransformFriendlyName')
    def getTransformFriendlyName(self, name):
        """Returns a friendly name for the given transform."""
        return self.transforms[name]['friendlyName']

    security.declareProtected(View, 'getEnabledTransforms')
    def getEnabledTransforms(self):
        """Returns list of names for enabled transforms"""
        return [name for name in self.transforms.keys() if self.transforms[name]['enabled']]

    security.declareProtected(View, 'performCommentTransform')
    def performCommentTransform(self, orig, **kwargs):
        """This performs the comment transform - also used for preview."""
        transform_tool = getToolByName(self, 'portal_transforms')

        # This one is very important, because transform object has no
        # acquisition context inside it, so we need to pass it our one
        context=kwargs.get('context', self)

        data = transform_tool._wrap('text/plain')

        for transform in self.getEnabledTransforms():
            data = transform_tool.convert(transform, orig, data, context)
            orig = data.getData()

        orig = orig.replace('\n', '<br/>')
        return orig

    # File upload - should be in a View once we get formcontroller support in Views
    security.declareProtected(View, 'getUploadedFiles')
    def getUploadedFiles(self):
        request = self.REQUEST

        result = []
        files = request.get('files', [])

        if not files:
            return []

        sdm = getToolByName(self, 'session_data_manager', None)

        if sdm is not None:
            pt = getToolByName(self, 'plone_utils')
            hassession = sdm.hasSessionData()

            for file in files:
                if isinstance(file, basestring) and hassession:
                    # Look it up from session
                    oldfile = request.SESSION.get(file, None)
                    if oldfile is not None:
                        result.append(oldfile)
                if isinstance(file, FileUpload):
                    if file:
                        filename = file.filename.split('\\')[-1]
                        id = pt.normalizeString(filename)
                        ct = file.headers.getheader('content-type')
                        if ct is None:
                            ct = ''
                        newfile = File(id, id, file, ct)
                        request.SESSION[id] = newfile
                        result.append(newfile)

            # delete files form session if not referenced
            new_filelist = [x.getId() for x in result]
            old_filelist = hassession and request.SESSION.get('ploneboard_uploads', []) or []
            for removed in [f for f in old_filelist if f not in new_filelist]:
                del request.SESSION[f]
            if hassession or new_filelist:
                request.SESSION['ploneboard_uploads'] = new_filelist

        return result

    security.declareProtected(View, 'clearUploadedFiles')
    def clearUploadedFiles(self):
        # Get previously uploaded files with a reference in request
        # + files uploaded in this request
        # XXX Add variable to keep track of filenames?
        request = self.REQUEST

        sdm = getToolByName(self, 'session_data_manager', None)

        if sdm is not None:
            if sdm.hasSessionData():
                old_filelist = request.SESSION.get('ploneboard_uploads', None)
                if old_filelist is not None:
                    for file in old_filelist:
                        if request.SESSION.has_key(file):
                            del request.SESSION[file]
                    del request.SESSION['ploneboard_uploads']


Globals.InitializeClass(PloneboardTool)
registerToolInterface(PLONEBOARD_TOOL, IPloneboardTool)



from zope.interface import implements

from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName

from Products.Archetypes.public import BaseBTreeFolderSchema, Schema, TextField, LinesField
from Products.Archetypes.public import BaseBTreeFolder, registerType
from Products.Archetypes.public import TextAreaWidget, LinesWidget

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.Ploneboard.config import PROJECTNAME
from Products.Ploneboard.permissions import ViewBoard, SearchBoard, \
    AddForum, ManageBoard
from Products.Ploneboard.content.PloneboardForum import PloneboardForum
from Products.Ploneboard.interfaces import IPloneboard
from Products.Ploneboard import utils


schema = BaseBTreeFolderSchema + Schema((
    TextField('description',
              searchable = 1,
              default_content_type = 'text/html',
              default_output_type = 'text/plain',
              widget = TextAreaWidget(description = "Enter a brief description of the board.",
                                      description_msgid = "help_description_board",
                                      i18n_domain = "ploneboard",
                                      label = "Description",
                                      label_msgid = "label_description_board",
                                      rows = 5)),
    LinesField('categories',
               widget = LinesWidget(
                   description = "Enter the categories you want to have available for forums, one category on each line.",
                   description_msgid = "help_categories_board",
                   label = "Categories",
                   label_msgid = "label_categories_board",
                   i18n_domain = "ploneboard")),
    ))

utils.finalizeSchema(schema)


class Ploneboard(BrowserDefaultMixin, BaseBTreeFolder):
    """Ploneboard is the outmost board object, what shows up in your site."""
    implements(IPloneboard)
    __implements__ = (BrowserDefaultMixin.__implements__, BaseBTreeFolder.__implements__,)

    meta_type = 'Ploneboard'

    schema = schema

    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    def getCatalog(self):
        return getToolByName(self, 'portal_catalog')

    security.declareProtected(ManageBoard, 'edit')
    def edit(self, **kwargs):
        """Alias for update()
        """
        self.update(**kwargs)

    security.declareProtected(AddForum, 'addForum')
    def addForum( self
                , id
                , title
                , description ):
        """Add a forum to the board.

        XXX: Should be possible to parameterise the exact type that is being
        added.
        """
        kwargs = {'title' : title, 'description' : description}
        forum = PloneboardForum(id)
        self._setObject(id, forum)
        forum = getattr(self, id)
        forum.initializeArchetype(**kwargs)
        forum._setPortalTypeName('PloneboardForum')
        forum.notifyWorkflowCreated()
        # Enable topic syndication by default
        syn_tool = getToolByName(self, 'portal_syndication', None)
        if syn_tool is not None:
            if (syn_tool.isSiteSyndicationAllowed() and not syn_tool.isSyndicationAllowed(forum)):
                syn_tool.enableSyndication(forum)

        #forum.setDescription(description)
        return forum

    security.declareProtected(ViewBoard, 'getForums')
    def getForums(self, sitewide=False):
        """Return all the forums in this board."""
        query = {'object_provides':'Products.Ploneboard.interfaces.IForum'}
        if not sitewide:
            query['path'] = '/'.join(self.getPhysicalPath())
        return [f.getObject() for f in self.getCatalog()(query)]


    security.declareProtected(ViewBoard, 'getForumIds')
    def getForumIds(self):
        """Return all the forums in this board."""
        return [f.getId for f in self.getCatalog()(
                object_provides='Products.Ploneboard.interfaces.IForum')]

    security.declareProtected(ManageBoard, 'removeForum')
    def removeForum(self, forum_id):
        """Remove a forum from this board."""
        self._delObject(forum_id)

    security.declareProtected(SearchBoard, 'searchComments')
    def searchComments(self, query):
        """This method searches through all forums, conversations and comments."""
        return self.getCatalog()(**query)

    security.declarePublic('getForum')
    def getForum(self, forum_id):
        """Returns forum with specified forum id."""
        #return getattr(self, forum_id, None)
        forums = self.getCatalog()(
                object_provides='Products.Ploneboard.interfaces.IForum',
                getId=forum_id)
        if forums:
            return forums[0].getObject()
        else:
            return None

    def __nonzero__(self):
        return 1

registerType(Ploneboard, PROJECTNAME)



"""Migrate from 0.1b1 to 1.0b.
"""
# Zope imports
from ZODB.PersistentMapping import PersistentMapping
from Acquisition import aq_base

# CMF imports
from Products.CMFCore.utils import getToolByName

# Product imports


class Migration(object):
    """Migrate from 0.1b1 to 1.0b.
    """

    def __init__(self, site, out):
        self.site = site
        self.out = out
        self.catalog = getToolByName(self.site, 'portal_catalog')

    def migrate(self):
        """Run migration on site object passed to __init__.
        """
        print >> self.out, u"Migrating Ploneboard 0.1b1 -> 1.0b"
        self.findAndCatalogAndClean()

    def findAndCatalogAndClean(self):
        """Manually find all the ploneboard types, recatalog them, and remove
        their old index objects that are just ZODB turds now.
        """
        # Ploneboard instances themselves should still be cataloged...
        pb_brains = self.catalog(portal_type='Ploneboard')
        forum_count = 0
        conv_count = 0
        comm_count = 0
        for pb in [brain.getObject() for brain in pb_brains]:
            if pb.hasObject('ploneboard_catalog'):
                pb._delObject('ploneboard_catalog')
                msg = u"Removed stale 'ploneboard_catalog' from Ploneboard at %s."
                print >> self.out, msg % '/'.join(pb.getPhysicalPath())
            else:
                msg = u"Checked for stale 'ploneboard_catalog' object on %s, but not present."
                print >> self.out, msg % '/'.join(pb.getPhysicalPath())
            # Directly access the stored forums.  The Ploneboard API won't work here
            for forum in pb.objectValues('PloneboardForum'):
                forum.reindexObject()
                self._cleanIndex(forum)
                forum_count += 1
                for conv in forum.objectValues('PloneboardConversation'):
                    conv.reindexObject()
                    self._cleanIndex(conv)
                    conv_count += 1
                    for comm in conv.objectValues('PloneboardComment'):
                        comm.reindexObject()
                        comm_count += 1
        msg = "Indexed and cleaned %s forums, %s conversations, and %s comments."
        print >> self.out, msg % (forum_count, conv_count, comm_count)

    def _cleanIndex(self, ob):
        if getattr(aq_base(ob), '_index', None) is not None:
            delattr(ob, '_index')
            msg = u"Removed stale '_index' from %s at %s."
            print >> self.out, msg % (ob.meta_type, '/'.join(ob.getPhysicalPath()))
        else:
            msg = u"Checked for stale '_index' attribute on '%s' at %s, but not present."
            print >> self.out, msg % (ob.meta_type, '/'.join(ob.getPhysicalPath()))




#
# Comment tests
#

import unittest
from Products.Ploneboard.tests import PloneboardTestCase
from Products.CMFPlone.utils import _createObjectByType

class TestITextContentAdapter(PloneboardTestCase.PloneboardTestCase):

    def afterSetUp(self):
        from Products.ATContentTypes.interface import ITextContent 
        self.board = _createObjectByType('Ploneboard', self.folder, 'board')
        self.forum = _createObjectByType('PloneboardForum', self.board, 'forum')
        self.conv = self.forum.addConversation('conv1', 'conv1 body')
        self.comment = self.conv.addComment("c1 title", "c1 body")
        self.textContent = ITextContent(self.comment)
        

    def testGetText(self):
        self.assertEqual(self.comment.getText(), 
                         self.textContent.getText())
    
    def testSetText(self):
        s = 'blah'
        self.textContent.setText('blah')
        
        self.assertEqual(self.comment.getText(), s)
        self.assertEqual(self.textContent.getText(), s)
        
    def testCookedBody(self):
        self.assertEqual(self.textContent.CookedBody(), 
                         self.comment.getText())

    def testEditableBody(self):
        self.assertEqual(self.textContent.CookedBody(), 
                         self.comment.getRawText())


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestITextContentAdapter))
    return suite



#
# Ploneboard tests
#

import unittest
from Products.Ploneboard.tests import PloneboardTestCase

from Products.Ploneboard.batch import Batch

from Products.CMFPlone.utils import _createObjectByType 

class TestBatch(PloneboardTestCase.PloneboardTestCase):

    def afterSetUp(self):
        self.board = _createObjectByType('Ploneboard', self.folder, 'board')
        self.forum = _createObjectByType('PloneboardForum', self.board, 'forum')
        
    def batch(self, size=5, start=0, orphan=1):
        return Batch(self.forum.getConversations, 
                     self.forum.getNumberOfConversations(), 
                     size, start, orphan=orphan)
        
    def sorted(self, b):
        items = [x for x in b]
        items.sort(lambda x, y: cmp(x.Title(), y.Title()))
        return items
        
    def sbatch(self, size=5, start=0, orphan=1):
        b = self.batch(size, start, orphan)
        return b, self.sorted(b)
        
    def testEmptyBatch(self):
        b = self.batch()
        self.assertEqual(len(b), 0)
        self.assertEqual(b.next, None)
        self.assertEqual(b.previous, None)

    def testLessThanOnePage(self):
        for i in range(3):
            self.forum.addConversation('Title %02s' % i)
        b, items = self.sbatch()
        self.assertEqual(len(b), 3)
        for i in range(3):
            self.assertEqual(items[i].Title(), 'Title %02s' % i)
        self.assertEqual(b.next, None)
        self.assertEqual(b.previous, None)
        
    def testExactlyOnePage(self):
        for i in range(5):
            self.forum.addConversation('Title %02s' % i)
        b, items = self.sbatch()
        self.assertEqual(len(b), 5)
        for i in range(5):
            self.assertEqual(items[i].Title(), 'Title %02s' % i)
        self.assertEqual(b.next, None)
        self.assertEqual(b.previous, None)
        
    def testOnePagePlusOrphan(self):
        for i in range(6):
            self.forum.addConversation('Title %02s' % i)
        b, items = self.sbatch()
        self.assertEqual(len(b), 6)
        for i in range(6):
            self.assertEqual(items[i].Title(), 'Title %02s' % i)
        self.assertEqual(b.next, None)
        self.assertEqual(b.previous, None)
        
    # note: when testing more than one page, we don't test for the 
    # exact objects returned, because getConversations returns things
    # sorted by modified date, but when we create objects in a loop like 
    # below, the resolution of the timestamp isn't good enough, and thus order
    # is sometimes unpredictable, leading to non-deterministic tests.
        
    def testOnePagePlusOneMoreThanOrphan(self):
        for i in range(7):
            self.forum.addConversation('Title %02s' % i)
        b = self.batch()
        self.assertEqual(b.previous, None)
        self.assertNotEqual(b.next, None)
        self.assertEqual(len(b.next), 2)
    
    def testGetLastPage(self):
        for i in range(8):
            self.forum.addConversation('Title %02s' % i)
        b = self.batch(start=5)
        self.assertEqual(len(b), 3)
        self.assertEqual(b.next, None)
        self.assertNotEqual(b.previous, None)
        self.assertEqual(len(b.previous), 5)
        
    def testGetMiddlePage(self):
        for i in range(12):
            self.forum.addConversation('Title %02s' % i)
        b = self.batch(start=5)
        self.assertEqual(len(b), 5)
        self.assertNotEqual(b.next, None)
        self.assertNotEqual(b.previous, None)
        self.assertEqual(len(b.next), 2)
        self.assertEqual(len(b.previous), 5)
        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestBatch))
    return suite



from zope.interface import implements
from zope import schema
from zope.component import getUtility
from zope.component import getMultiAdapter
from zope.formlib.form import Fields
from plone.memoize.view import memoize
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.i18n.normalizer.interfaces import IIDNormalizer

from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from Products.Ploneboard.utils import PloneboardMessageFactory as _


class IRecentConversationsPortlet(IPortletDataProvider):
    """A portlet which shows recent Ploneboard conversations.
    """


    title = schema.TextLine(title=_(u"title_title",
                                default=u"Portlet title"),
                        required=True,
                        default=u"Recent messages")

    count = schema.Int(title=_(u"title_count",
                                default=u"Number of items to display"),
                       description=_(u"help_count",
                                default=u"How many items to list."),
                       required=True,
                       default=5)

class Assignment(base.Assignment):
    implements(IRecentConversationsPortlet)

    title = u"Recent messages"
    count = 5

    def __init__(self, title=None, count=None):
        if title is not None:
            self.title=title
        if count is not None:
            self.count=count


class Renderer(base.Renderer):
    def __init__(self, context, request, view, manager, data):
        base.Renderer.__init__(self, context, request, view, manager, data)

    @memoize
    def results(self):
        ct=getToolByName(self.context, "portal_catalog")
        normalize=getUtility(IIDNormalizer).normalize
        icons=getMultiAdapter((self.context, self.request),
                                name="plone").icons_visible()
        if icons:
            portal=getMultiAdapter((self.context, self.request),
                                    name="plone_portal_state").portal_url()+"/"
        brains=ct(
                object_provides="Products.Ploneboard.interfaces.IConversation",
                sort_on="modified",
                sort_order="reverse",
                sort_limit=self.data.count)[:self.data.count]

        def morph(brain):
            obj=brain.getObject()
            forum=obj.getForum()

            return dict(
                    title = brain.Title,
                    description = brain.Description,
                    url = brain.getURL()+"/view",
                    icon = icons and portal+brain.getIcon or None,
                    forum_url = forum.absolute_url(),
                    forum_title = forum.title_or_id(),
                    review_state = normalize(brain.review_state),
                    portal_type = normalize(brain.portal_type))

        return [morph(brain) for brain in brains]

    @property
    def available(self):
        return len(self.results())>0

    def update(self):
        self.conversations=self.results()

    @property
    def title(self):
        return self.data.title

    @property
    def next_url(self):
        state=getMultiAdapter((self.context, self.request),
                                name="plone_portal_state")
        return state.portal_url()+"/ploneboard_recent"

    render = ViewPageTemplateFile("recent.pt")


class AddForm(base.NullAddForm):
    form_fields = Fields(IRecentConversationsPortlet)
    label = _(u"label_add_portlet",
                default=u"Add recent conversations portlet.")
    description = _(u"help_add_portlet",
            default=u"This portlet shows conversations with recent comments.")

    def create(self, data):
        return Assignment(title=data.get("title"), count=data.get("count"))


class EditForm(base.EditForm):
    form_fields = Fields(IRecentConversationsPortlet)
    label = _(u"label_add_portlet",
                default=u"Add recent conversations portlet.")
    description = _(u"help_add_portlet",
            default=u"This portlet shows conversations with recent comments.")




### Register Transforms
### This is interesting because we don't expect all transforms to be
### available on all platforms. To do this we allow things to fail at
### two levels
### 1) Imports
###    If the import fails the module is removed from the list and
###    will not be processed/registered
### 2) Registration
###    A second phase happens when the loaded modules register method
###    is called and this produces an instance that will used to
###    implement the transform, if register needs to fail for now it
###    should raise an ImportError as well (dumb, I know)

from Products.PortalTransforms.libtransforms.utils import MissingBinary
modules = [
    'text_to_emoticons',
    'url_to_hyperlink',
    ]

g = globals()
transforms = []
for m in modules:
    try:
        ns = __import__(m, g, g, None)
        transforms.append(ns.register())
    except ImportError, e:
        print "Problem importing module %s : %s" % (m, e)
    except MissingBinary, e:
        print e
    except:
        import traceback
        traceback.print_exc()


def initialize(engine):
    for transform in transforms:
        engine.registerTransform(transform)



#
# Forum tests
#

import unittest
from zope.interface.verify import verifyClass, verifyObject

import PloneboardTestCase
from Products.Ploneboard.interfaces import IForum
from Products.Ploneboard.content.PloneboardForum import PloneboardForum

from Products.CMFPlone.utils import _createObjectByType


class TestPloneboardForum(PloneboardTestCase.PloneboardTestCase):

    def afterSetUp(self):
        self.board = _createObjectByType('Ploneboard', self.folder, 'board')
        self.forum = _createObjectByType('PloneboardForum', self.board, 'forum')

    def testInterfaceVerification(self):
        self.failUnless(verifyClass(IForum, PloneboardForum))

    def testInterfaceConformance(self):
        self.failUnless(IForum.providedBy(self.forum))
        self.failUnless(verifyObject(IForum, self.forum))        

    def testForumFields(self):
        """
        Check the fields on Forum, especially the Description field mimetypes.
        """
        forum = self.forum
        self.assertEqual(forum.getId(), 'forum')
        forum.setTitle('title')
        self.assertEqual(forum.Title(), 'title')
        forum.setDescription('description')
        self.assertEqual(forum.Description(), 'description')
        self.assertEqual(forum.Description(mimetype='text/html'), '<p>description</p>')

    def testForumCategory(self):
        self.failUnlessEqual(len(self.forum.getCategories()), 0)
        self.board.setCategories(['Category'])
        self.failUnlessEqual(len(self.forum.getCategories()), 1)

    # Interface tests

    def testGetBoard(self):
        self.failUnlessEqual(self.board, self.forum.getBoard())

    def testGetBoardOutsideStrictContainment(self):
        forum = _createObjectByType('PloneboardForum', self.folder, 'forum')
        self.failUnlessEqual(None, forum.getBoard())

    def testAddConversation(self):
        """
        Create new folder in home directory & check its basic properties and behaviour
        """
        forum = self.forum
        conv = forum.addConversation('subject', 'body')
        conv_id = conv.getId()
        self.failUnless(conv_id in forum.objectIds())
        self.assertEqual(conv.Title(), 'subject')

    def testGetConversation(self):
        forum = self.forum
        conv = forum.addConversation('subject', 'body')
        self.failUnlessEqual(conv, forum.getConversation(conv.getId()))

    def testGetConversationOutsideStrictContainment(self):
        # Make a folder inside the forum, then a conversation in the folder
        pass

    def testRemoveConversation(self):
        """
        Create new folder in home directory & check its basic properties and behaviour
        """
        forum = self.forum
        conv = forum.addConversation('subject', 'body')
        conv_id = conv.getId()
        forum.removeConversation(conv_id)
        self.assertEqual(len(forum.objectIds()), 0)
        self.failIf(conv_id in forum.objectIds())

    def testGetConversations(self):
        forum = self.forum
        conv = forum.addConversation('subject', 'body')
        conv2 = forum.addConversation('subject2', 'body2')
        # Notice reverse ordering, last always first
        self.failUnlessEqual(forum.getConversations(), [conv2, conv]) 
        # Check to make sure it doesn't get comments elsewhere
        forum2 = _createObjectByType('PloneboardForum', self.board, 'forum2')
        forum2.addConversation('subject', 'body')
        self.failUnlessEqual(forum.getConversations(), [conv2, conv]) 

    def testGetConversationsWithSlicing(self):
        forum = self.forum
        conv = forum.addConversation('subject', 'body')
        conv2 = forum.addConversation('subject2', 'body2')
        self.failUnlessEqual(forum.getConversations(limit=1, offset=0), [conv2]) 
        self.failUnlessEqual(forum.getConversations(limit=1, offset=1), [conv]) 

    def testGetConversationsOutsideStrictContainment(self):
        # Make a folder inside the forum, then a conversation in the folder
        pass

    def testGetNumberOfConversations(self):
        forum = self.forum
        self.failUnlessEqual(forum.getNumberOfConversations(), 0)
        conv = forum.addConversation('subject', 'body')
        self.failUnlessEqual(forum.getNumberOfConversations(), 1)
        conv2 = forum.addConversation('subject2', 'body2')
        self.failUnlessEqual(forum.getNumberOfConversations(), 2)
        forum.removeConversation(conv.getId())
        self.failUnlessEqual(forum.getNumberOfConversations(), 1)
        # Check to make sure it doesn't count conversations elsewhere
        forum2 = _createObjectByType('PloneboardForum', self.board, 'forum2')
        conv = forum2.addConversation('subject', 'body')
        self.failUnlessEqual(forum.getNumberOfConversations(), 1)

    def testGetNumberOfComments(self):
        forum = self.forum
        self.failUnlessEqual(forum.getNumberOfComments(), 0)
        conv = forum.addConversation('subject', 'body')
        self.failUnlessEqual(forum.getNumberOfComments(), 0)
        conv2 = forum.addConversation('subject2', 'body2')
        self.failUnlessEqual(forum.getNumberOfComments(), 0)
        forum.removeConversation(conv.getId())
        self.failUnlessEqual(forum.getNumberOfComments(), 0)
        conv2.addComment('followup', 'text')
        self.failUnlessEqual(forum.getNumberOfComments(), 1)
        # Check to make sure it doesn't count comments elsewhere
        forum2 = _createObjectByType('PloneboardForum', self.board, 'forum2')
        conv = forum2.addConversation('subject', 'body')
        conv.addComment("another", "another")
        self.failUnlessEqual(forum.getNumberOfComments(), 1)
    
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestPloneboardForum))
    return suite



import Products.Five
import Products.ATContentTypes

from DateTime import DateTime

def addMember(self, username, fullname="", email="", roles=('Member',), last_login_time=None):
    self.portal.portal_membership.addMember(username, 'secret', roles, [])
    member = self.portal.portal_membership.getMemberById(username)
    member.setMemberProperties({'fullname': fullname, 'email': email,
                                'last_login_time': DateTime(last_login_time),})

def setUpDefaultMembersBoardAndForum(self):
    addMember(self, 'member1', 'Member one', roles=('Member',))
    addMember(self, 'member2', 'Member two', roles=('Member',))
    addMember(self, 'manager1', 'Manager one', roles=('Manager',))
    addMember(self, 'reviewer1', 'Manager one', roles=('Reviewer',))

    self.workflow = self.portal.portal_workflow

    self.setRoles(('Manager',))
    self.portal.invokeFactory('Ploneboard', 'board1')
    self.board = self.portal.board1
    self.forum = self.board.addForum('forum1', 'Forum 1', 'Forum one')
    self.setRoles(('Member',))

def disableScriptValidators(portal):
    from Products.CMFFormController.FormController import ANY_CONTEXT, ANY_BUTTON
    scripts = ['add_comment_script', 'add_conversation_script', 'add_forum_script']
    try:
        for v in portal.portal_skins.ploneboard_scripts.objectValues():
            if v.id in scripts:
                v.manage_doCustomize('custom')
                portal.portal_form_controller.addFormValidators(v.id, ANY_CONTEXT, ANY_BUTTON, [])
    except:
        pass

def logoutThenLoginAs(self, browser, userid):
    browser.open('%s/logout' % self.portal.absolute_url())
    browser.open('%s/login_form' % self.portal.absolute_url())
    browser.getControl(name='came_from').value = self.portal.absolute_url()
    browser.getControl(name='__ac_name').value = userid
    browser.getControl(name='__ac_password').value = 'secret'
    browser.getControl('Log in').click()
    return



"""
$Id: __init__.py 55896 2007-12-19 14:36:54Z wichert $
"""

from Products.Archetypes.public import process_types, listTypes
from Products.CMFCore.DirectoryView import registerDirectory
from Products.Ploneboard.PloneboardTool import PloneboardTool

import sys

from Products.Ploneboard.config import SKINS_DIR, GLOBALS, PROJECTNAME
import Products.Ploneboard.catalog

registerDirectory(SKINS_DIR, GLOBALS)

this_module = sys.modules[ __name__ ]

def initialize(context):
    ##Import Types here to register them
    import Products.Ploneboard.content

    # If we put this import line to the top of module then
    # utils will magically point to Ploneboard.utils
    from Products.CMFCore import utils
    utils.ToolInit('Ploneboard Tool', 
            tools=(PloneboardTool,), 
            icon='tool.gif'
            ).initialize(context)

    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)

    # Assign an own permission to all content types
    # Heavily based on Bricolite's code from Ben Saller
    import permissions as perms

    allTypes = zip(content_types, constructors)
    for atype, constructor in allTypes:
        kind = "%s: %s" % (PROJECTNAME, atype.archetype_name)
        utils.ContentInit(
            kind,
            content_types      = (atype,),
            # Add permissions look like perms.Add{meta_type}
            permission         = getattr(perms, 'Add%s' % atype.meta_type),
            extra_constructors = (constructor,),
            fti                = ftis,
            ).initialize(context)

              
    from AccessControl import allow_class      
    from batch import Batch
    allow_class(Batch)
    this_module.Batch = Batch

# Avoid breaking old Ploneboard instances when moving content types modules
# from Ploneboard/types/ to Ploneboard/content/
import content
sys.modules['Products.Ploneboard.types'] = content



## Script (Python) "getKeyedForums"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=forums=None
##title=Get forums keyed on category
##

if forums is None:
    forums = context.getForums()

result = {}

for forum in forums:
    if hasattr(forum, 'getCategory'):
        categories = forum.getCategory()
        if not categories:
            categories = None
        if not same_type(categories, ()) and not same_type(categories, []):
            categories = categories,
        for category in categories:
            try:
                categoryforums = result.get(category, [])
                categoryforums.append(forum)
                result[category] = categoryforums
            except TypeError: # category is list?!
                result[', '.join(category)] = forum
return result


from Products.PortalTransforms.interfaces import itransform
from ZODB.PersistentMapping import PersistentMapping
from Products.CMFCore.utils import getToolByName
from Products.Ploneboard.utils import TransformDataProvider
import re
import copy

class EmoticonDataProvider(TransformDataProvider):
    def __init__(self):
        """ We need to redefine config and config_metadata
        in base class.
        """
        self.config = PersistentMapping()
        self.config_metadata = PersistentMapping()
        
        self.config.update({ 'inputs' : self.defaultEmoticons()})

        self.config_metadata.update({
            'inputs' : {
                'key_label' : 'emoticon code', 
                'value_label' : 'image name', 
                'description' : 'Emoticons to images mapping'}
            })

    def defaultEmoticons():
        emoticons = { ':)' : '<img src="smiley_smile.png" alt=":)" title="Smile" />'
                    , ':(' : '<img src="smiley_sad.png" alt=":(" title="Sad" />'
                    , '8-)' : '<img src="smiley_cool.png" alt="8)" title="Cool" />'
                    , ':D' : '<img src="smiley_lol.png" alt=":D" title="Big grin" />'
                    , ':|' : '<img src="smiley_skeptic.png" alt=":|" title="Skeptic" />'
                    , ':o' : '<img src="smiley_surprised.png" alt=":o" title="Surprised" />'
                    , ':P' : '<img src="smiley_tongue.png" alt=":P" title="Tongue-in-cheek" />'
                    , ';)' : '<img src="smiley_wink.png" alt=";)" title="Wink" />'
                    , ':-)' : '<img src="smiley_smile.png" alt=":)" title="Smile" />'
                    , ':-(' : '<img src="smiley_sad.png" alt=":(" title="Sad" />'
                    , ':-D' : '<img src="smiley_lol.png" alt=":D" title="Big grin" />'
                    , ':-|' : '<img src="smiley_skeptic.png" alt=":|" title="Skeptic" />'
                    , ':-o' : '<img src="smiley_surprised.png" alt=":o" title="Surprised" />'
                    , ':-P' : '<img src="smiley_tongue.png" alt=":P" title="Tongue-in-cheek" />'
                    , ';-)' : '<img src="smiley_wink.png" alt=";)" title="Wink" />'
                    }
        return emoticons
    defaultEmoticons=staticmethod(defaultEmoticons)

def registerDataProvider():
    return EmoticonDataProvider()


class TextToEmoticons:
    """transform which replaces text emoticons into urls to emoticons images"""

    __implements__ = itransform

    __name__ = "text_to_emoticons"
    output = "text/plain"

    def __init__(self, name=None, inputs=('text/plain',)):
        self.config = { 'inputs' : inputs, }
        self.config_metadata = {
            'inputs' : ('list', 'Inputs', 'Input(s) MIME type. Change with care.'),
            }
        if name:
            self.__name__ = name
            
    def name(self):
        return self.__name__

    def __getattr__(self, attr):
        if attr == 'inputs':
            return self.config['inputs']
        if attr == 'output':
            return self.config['output']
        raise AttributeError(attr)

    def convert(self, orig, data, **kwargs):
        """ Replace in 'orig' all occurences of any key in the given
          dictionary by its corresponding value.  Returns the new string.
        """
        # Get acquisition context
        context = kwargs.get('context')
        
        url_tool = getToolByName(context, 'portal_url')
        dict = EmoticonDataProvider.defaultEmoticons()
        
        # Here we need to find relative images path to the root of site
        # This is done for image cashing.
        dictionary = copy.deepcopy(dict)
        
        rev_dict = {}
        for k, v in dict.items():
            rev_dict[v] = k
        obj_ids = tuple(dict.values())
        # To speed up search we are narrowing search path
        start_path = url_tool.getPortalPath() + '/portal_skins'
        start_obj = context.restrictedTraverse(start_path + '/custom')
        results = context.PrincipiaFind(start_obj, obj_ids=obj_ids, obj_metatypes=('Image', ), search_sub=1)
        start_obj = context.restrictedTraverse(start_path + '/ploneboard_templates')
        results.extend(context.PrincipiaFind(start_obj, obj_ids=obj_ids, obj_metatypes=('Image', ), search_sub=1))
        for rec in results:
            obj = rec[1]
            dictionary[rev_dict[obj.getId()]] = '<img src="%s" />' % url_tool.getRelativeContentURL(obj)
        
        #based on ASPN recipe - http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/81330
        # Create a regular expression  from the dictionary keys
        regex = re.compile("(%s)(?!\")" % "|".join(map(re.escape, dictionary.keys())))
        # For each match, look-up corresponding value in dictionary
        newdata = regex.sub(lambda mo, d=dictionary: d[mo.string[mo.start():mo.end()]], orig) 
        data.setData(newdata)
        return data

def register():
    return TextToEmoticons()



from Products.PortalTransforms.interfaces import itransform
import re

hider = "##HIDE"
schemematcher = re.compile ("(mailto|telnet|gopher|http|https|ftp)", re.I)
hiddenschemematcher = re.compile ("(mailto|telnet|gopher|http|https|ftp)" + hider, re.I)
elementmatcher = re.compile("<[^>]+>")
emailRegexp = re.compile(r'["=]?(\b[A-Z0-9._%+=?\*^-]+@[A-Z0-9._%-]+\.[A-Z]{2,4}\b)', re.I|re.S|re.U)
urlmatcher = re.compile(
                    r"\b(?P<url>(?P<scheme>http|https|ftp|telnet|mailto|gopher):(?P<interfix>//)"
                    r"(?:(?P<login>(?P<username>[a-zA-Z0-9]+)(?::(?P<password>[A-Za-z0-9]+))?)@)?"
                    r"(?P<hostname>[A-Za-z0-9.-]+(?::(?P<port>[0-9]+))?)"
                    r"(?P<path>[A-Za-z0-9@~_=?/.&;%#+-]*))", re.I)

class URLToHyperlink:
    """transform which replaces urls and email into hyperlinks"""

    __implements__ = itransform

    __name__ = "url_to_hyperlink"
    output = "text/plain"


    def __init__(self, name=None, inputs=('text/plain',)):
        self.config = { 'inputs' : inputs, }
        self.config_metadata = {
            'inputs' : ('list', 'Inputs', 'Input(s) MIME type. Change with care.'),
            }
        if name:
            self.__name__ = name
            
    def name(self):
        return self.__name__

    def __getattr__(self, attr):
        if attr == 'inputs':
            return self.config['inputs']
        if attr == 'output':
            return self.config['output']
        raise AttributeError(attr)

    def linkify(input):
        def hidescheme(match):
            text=input[match.start():match.end()]
            text=text.replace("@", "@"+hider)
            return schemematcher.sub("\\1"+hider, text)

        def replaceEmail(match):
            url = match.groups()[0]
            return '<a href="mailto:%s">%s</a>' % (url, url)

        buf=elementmatcher.sub(hidescheme, input)
        buf=urlmatcher.sub(r'<a href="\1">\1</a>', buf)
        buf=emailRegexp.subn(replaceEmail, buf)[0]
        buf=buf.replace(hider, "")
        return buf
    linkify=staticmethod(linkify)

    def convert(self, orig, data, **kwargs):
        text = orig
        if not isinstance(text, unicode):
            text = unicode(text, 'utf-8', 'replace')
        
        text = self.linkify(text)
    
        data.setData(text.encode('utf-8'))
        return data

def register():
    return URLToHyperlink()

__all__ = [ "URLToHyperlink", "register" ]



#
# Ploneboard tests
#

from Products.Ploneboard.tests import PloneboardTestCase

# Catch errors in Install
from Products.Ploneboard.Extensions import Install

from Products.CMFCore.utils import getToolByName

class TestSetup(PloneboardTestCase.PloneboardTestCase):

    def testSkins(self):
        portal_skins = self.portal.portal_skins.objectIds()
        skins = (
            'ploneboard_images',
            'ploneboard_scripts',
            'ploneboard_templates',
        )
        for skin in skins:
            self.failUnless(skin in skins)

    def testPortalTypes(self):
        portal_types = self.portal.portal_types.objectIds()
        content_types = (
            'Ploneboard',
            'PloneboardForum',
            'PloneboardConversation',
            'PloneboardComment',
        )
        for content_type in content_types:
            self.failUnless(content_type in portal_types)

    def testTools(self):
        from Products.Ploneboard.config import PLONEBOARD_TOOL
        tool_names = (
            PLONEBOARD_TOOL,
            )
        for tool_name in tool_names:
            self.failUnless(tool_name in self.portal.objectIds())

    def testTransforms(self):
        from Products.Ploneboard.config import PLONEBOARD_TOOL
        tool = getToolByName(self.portal, PLONEBOARD_TOOL)
        transforms = [t for t in tool.getEnabledTransforms()]
        self.failUnless('safe_html' in transforms)
        self.failUnless('text_to_emoticons' in transforms)
        self.failUnless('url_to_hyperlink' in transforms)

    def testCatalogIndex(self):
        ct = getToolByName(self.portal, 'portal_catalog') 
        self.failUnless('object_provides' in ct.indexes())
        self.failUnless('num_comments' in ct.indexes())
        self.failUnless('num_comments' in ct.schema())

    def testPortalFactorySetup(self):
        portal_factory = getToolByName(self.portal, 'portal_factory') 
        factoryTypes = portal_factory.getFactoryTypes().keys()
        for t in ['Ploneboard', 'PloneboardComment', 'PloneboardConversation', 'PloneboardForum']:
            self.failUnless(t in factoryTypes)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestSetup))
    return suite



"""
$Id: interfaces.py 55896 2007-12-19 14:36:54Z wichert $
"""
# Dependency on Zope 2.8.x (or greater) or Five
from zope.interface import Interface, Attribute

class IPloneboard(Interface):
    """
    Ploneboard is the outmost board object, what shows up in your site.
    The board contains forums. Board is folderish. The number of items contained
    in Board should be limited and steady.
    This is an optional type.
    """

    def addForum(id, title, description):
        """
        The method add_forum takes id, title and description and creates a
        forum inside the board.
        Should this go away and rather just use the regular Plone content
        creation? That would make it easier to switch content types.
        """

    def removeForum(forum_id):
        """
        The method remove_forum removes the forum with the specified id from
        this board.
        """

    def getForum(forum_id):
        """
        Return the forum for forum_id, or None.
        """

    def getForumIds():
        """
        Returns the ids of the forums.
        If this is the only board in a site, it should return forum ids for
        the entire site, not just inside the board.
        """

    def getForums():
        """
        Return the forums
        If this is the only board in a site, it should return forums for the
        entire site, not just inside the board.
        """

    def searchComments(query):
        """
        This method searches through all forums, conversations and comments.
        """

class IForum(Interface):
    """
    A Forum contains conversations. Forum is folderish. The number of items contained
    in Forum is high and increases, so it is probably a good idea to use BTrees
    for indexing.
    """

    def getBoard():
        """
        Gets the containing board.
        Returns None if there are no boards in the site.
        """

    def addConversation(subject, body, **kw):
        """
        Adds a new conversation to the forum.
        Should this go away and rather just use the regular Plone content
        creation? That would make it easier to switch content types.
        """

    def getConversation(conversation_id):
        """
        Returns the conversation with the given conversation id.
        """

    def removeConversation(conversation_id):
        """
        Removes a conversation with the given conversation id from the forum.
        """

    def getConversations(limit=20, offset=0):
        """
        Returns a maximum of 'limit' conversations, the last updated conversations first,
        starting from 'offset'.
        """

    def getNumberOfConversations():
        """
        Returns the number of conversations in this forum.
        """

    def getNumberOfComments():
        """
        Returns the number of comments to this forum.
        """

class IConversation(Interface):
    """
    Conversation contains comments. The number of comments contained in
    Conversation is high and increases. It is recommended to use BTree for
    indexing and to autogenerate ids for contained comments.
    """

    def getForum():
        """
        Returns the containing forum.
        """

    def addComment(comment_subject, comment_body):
        """
        Adds a new comment with subject and body.
        """

    def getComment(comment_id):
        """
        Returns the comment with the specified id.
        """

    def getComments(limit=30, offset=0, **kw):
        """
        Retrieves the specified number of comments with offset 'offset'.
        In addition there are kw args for sorting and retrieval options.
        """

    def getNumberOfComments():
        """
        Returns the number of comments to this conversation.
        """

    def getLastCommentDate():
        """
        Returns a DateTime corresponding to the timestamp of the last comment
        for the conversation.
        """

    def getLastCommentAuthor():
        """
        Returns the author of the last comment for the conversation.
        """

    def getLastComment():
        """
        Returns the last comment as full object (no Brain).
        If there is no such one then None is returned
        """

    def getRootComments():
        """
        Return a list all comments rooted to the board; ie comments which
        are not replies to other comments.
        """

    def getFirstComment():
        """
        Returns the first (aka root) comment in this IConversation.
        """


class IComment(Interface):
    """
    A comment contains regular text body and metadata.
    """

    def getConversation():
        """
        Returns the containing conversation.
        """

    def addReply(comment_subject, comment_body):
        """
        Add a response to this comment of same type as object itself.
        """

    def inReplyTo():
        """
        Returns the comment object this comment is a reply to. If it is the
        topmost comment (ie: first comment in a conversation), it returns None.
        """

    def getReplies():
        """
        Returns the comments that were replies to this one.
        """

    def getTitle():
        """
        Returns the title of the comment.
        """

    def getText():
        """
        Returns the text of the comment.
        """

    def delete():
        """
        Delete this comment.  Will ensure to clean up any comments
        that were replies to this comment.
        """

class IAttachmentSupport(Interface):
    """
    Attachment support, typically for comments
    """
    def addAttachment(file, title=None):
        """
        Add a file attachment.
        """

    def hasAttachment():
        """
        Return 0 or 1 if this comment has attachments.
        """

    def getNumberOfAllowedAttachments():
        """
        Return the number of allowed attachments
        """

    def getNumberOfAttachments():
        """
        Return the number of attachments
        """
    def getAttachments():
        """
        Return all attachments
        """

class IPloneboardTool(Interface):
    """Services for Ploneboard: Handles text transformation plugins and attached files.
    """

    id = Attribute('id', 'Must be set to "portal_ploneboard"')

    def registerTransform(name, module, friendlyName=None):
        """Adds a text transformation module to portal_transforms.
        Used from the configuration panel
        """

    def unregisterTransform(name):
        """Removes the transformation module from portal_transforms
        Used from the configuration panel
        """

    def enableTransform(name, enabled=True):
        """Globally enables a transform (site wide)
        """

    def unregisterAllTransforms():
        """Removes from portal_transforms all transform modules added with Ploneboard
        """

    def getTransforms():
        """Returns list of transform names.
        """

    def getTransformFriendlyName(name):
        """Returns a friendly name for the given transform.
        """

    def getEnabledTransforms():
        """Returns list of names for enabled transforms.
        """

    def performCommentTransform(orig, **kwargs):
        """This performs the comment transform - also used for preview.
        """

    def getUploadedFiles():
        """Stores files from request in session and returns these files
        """

    def clearUploadedFiles():
        """Removes uploaded files from session machinery
        """



## Script (Python) "comment_redirect_to_conversation"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Redirect from a comment to it's conversation
##

# XXX if we ever do batching, we need extra logic here.
redirect_target = context.getConversation()
view = redirect_target.restrictedTraverse("@@plone_context_state").view_url()
anchor = context.getId()
  
response = context.REQUEST.get('RESPONSE', None)
if response is not None:
    response.redirect('%s#%s' % (view, anchor))
    print "Redirecting to %s" % redirect_target.absolute_url()
    return printed
    



from zope.interface import implements

from AccessControl import ClassSecurityInfo
from Acquisition import aq_inner, aq_chain
from OFS.Image import File

from Products.CMFCore.utils import getToolByName

from Products.Archetypes.public import BaseBTreeFolderSchema, Schema, TextField
from Products.Archetypes.public import BaseBTreeFolder, registerType
from Products.Archetypes.public import TextAreaWidget
from Products.Ploneboard.config import PROJECTNAME

from Products.CMFPlone.utils import _createObjectByType

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin
from Products.Ploneboard.permissions import (
    ViewBoard, AddComment, ManageConversation, EditComment)
from Products.Ploneboard.interfaces import IForum, IConversation

from Products.Ploneboard import utils

from Products.CMFPlone.interfaces.NonStructuralFolder \
    import INonStructuralFolder as ZopeTwoINonStructuralFolder
from Products.CMFPlone.interfaces.structure import INonStructuralFolder

PBConversationBaseBTreeFolderSchema = BaseBTreeFolderSchema.copy()
PBConversationBaseBTreeFolderSchema['title'].read_permission = ViewBoard
PBConversationBaseBTreeFolderSchema['title'].write_permission = EditComment

schema = PBConversationBaseBTreeFolderSchema + Schema((
    TextField('description',
              searchable = 1,
              read_permission = ViewBoard,
              write_permission = EditComment,
              default_content_type = 'text/plain',
              default_output_type = 'text/plain',
              widget = TextAreaWidget(description = "Enter a brief description of the conversation.",
                                      description_msgid = "help_description_conversation",
                                      label = "Description",
                                      label_msgid = "label_description_conversation",
                                      i18n_domain = "ploneboard",
                                      rows = 5)),
    ))
utils.finalizeSchema(schema)


class PloneboardConversation(BrowserDefaultMixin, BaseBTreeFolder):
    """Conversation contains comments."""

    implements(IConversation, INonStructuralFolder)
    __implements__ = (BaseBTreeFolder.__implements__, BrowserDefaultMixin.__implements__, ZopeTwoINonStructuralFolder)

    meta_type = 'PloneboardConversation'

    schema = schema

    _at_rename_after_creation = True


    security = ClassSecurityInfo()

    def getCatalog(self):
        return getToolByName(self, 'portal_catalog')

    security.declareProtected(ManageConversation, 'edit')
    def edit(self, **kwargs):
        """Alias for update()
        """
        self.update(**kwargs)

    security.declareProtected(ViewBoard, 'getTitle')
    def getTitle(self):
        """Get the title of this conversation"""
        return self.Title()

    security.declareProtected(ViewBoard, 'getForum')
    def getForum(self):
        """Returns containing forum."""
        # Try containment
        stoptypes = ['Plone Site']
        for obj in aq_chain(aq_inner(self)):
            if hasattr(obj, 'portal_type') and obj.portal_type not in stoptypes:
                if IForum.providedBy(obj):
                    return obj
        return None

    security.declareProtected(ManageConversation, 'removeComment')
    def removeComment( self, comment):
        self.manage_delObjects([comment.getId()])
        # XXX reparent replies to this comment ?

    security.declareProtected(AddComment, 'addComment')
    def addComment( self, title, text, creator=None, files=None):
        """Adds a new comment with subject and body."""
        id = self.generateId(prefix='')
        if not title:
            title = self.Title()


        m = _createObjectByType('PloneboardComment', self, id)

        # XXX: There is some permission problem with AT write_permission
        # and using **kwargs in the _createObjectByType statement.
        m.setTitle(title)
        m.setText(text)
        if creator is not None:
            m.setCreators([creator])

        # Create files in message
        if files:
            for file in files:
                # Get raw filedata, not persistent object with reference to tempstorage
                attachment = File(file.getId(), file.title_or_id(), str(file.data), file.getContentType())
                m.addAttachment(attachment)

        # If this comment is being added by anonymous, make sure that the true
        # owner in zope is the owner of the forum, not the parent comment or
        # conversation. Otherwise, that owner may be able to view or delete
        # the comment.
        membership = getToolByName(self, 'portal_membership')
        if membership.isAnonymousUser():
            forum = self.getForum()
            utils.changeOwnershipOf(m, forum.owner_info()['id'], False)

        self.reindexObject() # Sets modified
        return m

    security.declareProtected(ViewBoard, 'getComment')
    def getComment(self, comment_id, default=None):
        """Returns the comment with the specified id."""
        #return self._getOb(comment_id, default)
        comments = self.getCatalog()(
                object_provides='Products.Ploneboard.interfaces.IComment',
                getId=comment_id)
        if comments:
            return comments[0].getObject()
        else:
            return None

    security.declareProtected(ViewBoard, 'getComments')
    def getComments(self, limit=30, offset=0, **kw):
        """
        Retrieves the specified number of comments with offset 'offset'.
        In addition there are kw args for sorting and retrieval options.
        """
        query = {'object_provides' : 'Products.Ploneboard.interfaces.IComment',
                 'sort_on'           : 'created',
                 'sort_limit'        : (offset+limit),
                 'path'              : '/'.join(self.getPhysicalPath()),}
        query.update(kw)
        catalog=self.getCatalog()
        return [f.getObject() for f in catalog(**query)[offset:offset+limit]]

    security.declareProtected(ViewBoard, 'getNumberOfComments')
    def getNumberOfComments(self):
        """
        Returns the number of comments in this conversation.
        """
        return len(self.getCatalog()(
            object_provides='Products.Ploneboard.interfaces.IComment',
            path='/'.join(self.getPhysicalPath())))

    security.declareProtected(ViewBoard, 'getLastCommentDate')
    def getLastCommentDate(self):
        """
        Returns a DateTime corresponding to the timestamp of the last comment
        for the conversation.
        """
        comment = self.getLastComment()
        if comment:
            return comment.created()
        return None

    security.declareProtected(ViewBoard, 'getLastCommentAuthor')
    def getLastCommentAuthor(self):
        """
        Returns the name of the author of the last comment.
        """

        comment = self.getLastComment()
        if comment:
            return comment.Creator()
        return None

    security.declareProtected(ViewBoard, 'getLastComment')
    def getLastComment(self):
        """
        Returns the last comment as full object..
        Returns None if there is no comment
        """

        res = self.getCatalog()(
                object_provides='Products.Ploneboard.interfaces.IComment', \
                sort_on='created', sort_order='reverse', sort_limit=1,
                path='/'.join(self.getPhysicalPath()))
        if res:
            return res[0].getObject()
        return None

    security.declareProtected(ViewBoard, 'getRootComments')
    def getRootComments(self):
        """
        Return a list all comments rooted to the board; ie comments which
        are not replies to other comments.
        """
        raw = self.getComments()
        ours = [ comment for comment in raw if comment.inReplyToUID() is None]
        return ours

    security.declareProtected(ViewBoard, 'getFirstComment')
    def getFirstComment(self):
        """
        See IConversation.getFirstComment.__doc__
        """
        res = self.getCatalog()(
                object_provides='Products.Ploneboard.interfaces.IComment',
                sort_on='created', sort_limit=1,
                path='/'.join(self.getPhysicalPath()))
        if res:
            return res[0].getObject()
        else:
            return None

    security.declareProtected(ManageConversation, 'moveToForum')
    def moveToForum(self, forum_id):
        """Moves conversation to another forum"""
        forum = self.getForum().getBoard().getForum(forum_id)
        if forum:
            parent = self.getForum()
            cut_objects = parent.manage_cutObjects((self.getId(),) )
            forum.manage_pasteObjects(cut_objects)

    security.declareProtected(ManageConversation, 'delete')
    def delete(self):
        """"""
        parent = self.getForum()
        parent._delObject(self.getId())

    security.declareProtected(ViewBoard, 'Creator')
    def Creator(self):
        # XXX Backwards compatability with old version
        return getattr(self, '_creator', None) or BaseBTreeFolder.Creator(self)

    def __nonzero__(self):
        return 1

    # No setting of default page - makes no sense
    def canSetDefaultPage(self):
        return False

registerType(PloneboardConversation, PROJECTNAME)



#
# Ploneboard transform tests
#

import unittest
from Products.Ploneboard.tests import PloneboardTestCase
from Products.CMFCore.utils import getToolByName
from Products.Ploneboard.config import PLONEBOARD_TOOL

from Products.CMFPlone.utils import _createObjectByType


class TestTransformRegistration(PloneboardTestCase.PloneboardTestCase):
    """Test transform registration """

    def afterSetUp(self):
        self.board = _createObjectByType('Ploneboard', self.folder, 'board')

    def testDefaultRegistrations(self):
        """Check if the default registrations are present."""
        tool = getToolByName(self.portal, PLONEBOARD_TOOL)
        self.failUnlessEqual(len(tool.getTransforms()), 3)
        self.failUnlessEqual(len(tool.getEnabledTransforms()), 3)

    def testDisabling(self):
        """Try registering and unregistering a transform"""
        tool = getToolByName(self.portal, PLONEBOARD_TOOL)
        tool.enableTransform('safe_html', enabled=False)
        self.failIf('safe_html' in tool.getEnabledTransforms())
        tool.enableTransform('safe_html')
        self.failUnless('safe_html' in tool.getEnabledTransforms())

    def testUnregisteringAllRemovesOnlyThoseAdded(self):
        tool = getToolByName(self.portal, PLONEBOARD_TOOL)
        tool.unregisterAllTransforms()
        transforms = getToolByName(self.portal, 'portal_transforms')
        self.failIf('url_to_hyperlink' in transforms.objectIds())
        self.failIf('text_to_emoticons' in transforms.objectIds())
        self.failUnless('safe_html' in transforms.objectIds())

    def testUnregisteringIndividualRemovesOnlyThoseAdded(self):
        tool = getToolByName(self.portal, PLONEBOARD_TOOL)
        transforms = getToolByName(self.portal, 'portal_transforms')
        tool.unregisterTransform('url_to_hyperlink')
        self.failIf('url_to_hyperlink' in transforms.objectIds())
        tool.unregisterTransform('safe_html')
        self.failUnless('safe_html' in transforms.objectIds())

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestTransformRegistration))
    return suite




#
# Ploneboard tests
#

import unittest
from zope.testing import doctest
from zope.app.testing import placelesssetup

setup = placelesssetup.setUp
teardown = placelesssetup.tearDown

def test_suite():
    return unittest.TestSuite((
        doctest.DocTestSuite('Products.Ploneboard.browser',
                             setUp=setup, tearDown=teardown),))




## Script (Python) "list_pending_search.py"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=obj=None
##title=Searches for pending comments in moderated boards
# $Id: moderation_count_search.py 55896 2007-12-19 14:36:54Z wichert $

if obj is None:
    obj = context

query = {}
query['review_state'] = 'pending'
query['portal_type'] = 'Ploneboard Comment'
query['path'] = '/'+ '/'.join(obj.getPhysicalPath()[1:])

reqget = context.REQUEST.get

# FIXME: this function seems useless
def supplement_query(field, index_name=None, reqget=reqget, query=query):
    if not index_name: index_name = field
    val = reqget(field, None)
    if val:
        query[index_name] = val

return len(context.getInternalCatalog()(REQUEST=query))



## Script (Python) "moderation.py"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Searches for pending comments in moderated boards
##
# Search the catalog, and depending on where the script is called from, filter
# and massage results until it is suitable for displaying. The main concern is
# too large number of search results.
# Massaging the results means getting the objects, which might not be that bad
# as we are going to write to them anyway (moderate) soon.
# Returned results might be forum objects, conversation objects or comment objects
# if returned number of pending comments is less than 100: display everything
# if context is conversation, display everything
# if context is board, only display link to forums with more than 50 pending comments
# if context is forum, only display link to conversations with more than 10 pending comments
 
query = {}
query['sort_on'] = 'created'
query['review_state'] = 'pending'
query['portal_type'] = 'PloneboardComment'
query['path'] = '/'.join(context.getPhysicalPath())

reqget = context.REQUEST.get

def supplement_query(field, index_name=None, reqget=reqget, query=query):
    if not index_name: index_name = field
    val = reqget(field, None)
    if val:
        query[index_name] = val

catalogresult = context.portal_catalog(query)

if context.portal_type == 'PloneboardConversation' or len(catalogresult) < 100:
    return [r.getObject() for r in catalogresult]

result = []
if context.portal_type == 'Ploneboard':
    for forum in context.contentValues('PloneboardForum'):
        query['path'] = '/'+ '/'.join(forum.getPhysicalPath()[1:])
        forumresult = context.portal_catalog(query)
        if len(forumresult) > 50:
            result.append(forum)
        else:
            result.extend([r.getObject() for r in forumresult])
    return result

if context.portal_type == 'PloneboardForum':
    # We need a dynamic programming solution here as search for each 
    # conversation is prohibitively expensive
    forumid = context.getId()
    conversations = {} # {id, [commentbrains, ]}
    for item in catalogresult:
        pathlist = item.getPath().split('/')
        conversationid = pathlist[pathlist.index(forumid)+1]
        if conversations.has_key(conversationid):
            conversations[conversationid].append(item)
        else:
            conversations[conversationid] = [item]
    for key in conversations.keys():
        if len(conversations[key]) > 10:
            # Could use a wrapper to store the conversation + length of pending comment
            # queue to avoid catalog calls in the moderation_form template
            result.append(context.getConversation(key))
        else:
            result.extend([i.getObject() for i in conversations[key]])

return result


from zope.interface import implements

from Products.ATContentTypes.interface import ITextContent

class CommentTextContent(object):
    implements(ITextContent)

    def __init__(self, context):
        self.context = context

    def getText(self, **kwargs):
        return self.context.getText()

    def setText(self, value, **kwargs):
        self.context.setText(value, **kwargs)
        
    def CookedBody(self, stx_level='ignored'):
        return self.getText()

    def EditableBody(self):
        return self.getRawText()



#
# Tests the default workflow
#

from AccessControl.Permission import Permission
from Products.CMFPlone.tests import PloneTestCase

from Products.CMFCore.utils import _checkPermission as checkPerm
from Products.CMFPlone.utils import _createObjectByType

from Products.Ploneboard.Extensions import WorkflowScripts # Catch errors
from Products.Ploneboard.tests import PloneboardTestCase
from Products.Ploneboard import permissions

default_user = PloneTestCase.default_user


class TestCommentWorkflow(PloneboardTestCase.PloneboardTestCase):

    def afterSetUp(self):
        self.workflow = self.portal.portal_workflow
        self.board = _createObjectByType('Ploneboard', self.folder, 'board')
        self.forum = _createObjectByType('PloneboardForum', self.board, 'forum')
        self.conv = self.forum.addConversation('conv1', 'conv1 body')
        self.comment = self.conv.addComment("title", "body")

        self.portal.acl_users._doAddUser('member', 'secret', ['Member'], [])
        self.portal.acl_users._doAddUser('member2', 'secret', ['Member'], [])
        self.portal.acl_users._doAddUser('reviewer', 'secret', ['Reviewer'], [])
        self.portal.acl_users._doAddUser('manager', 'secret', ['Manager'], [])

    # Check allowed transitions

    def testAutoPublishMemberposting(self):

        self.login('member')
        self.failUnless(checkPerm(permissions.ApproveComment, self.forum))
        self.failUnless(checkPerm(permissions.ApproveComment, self.conv))
        self.failUnless(checkPerm(permissions.ApproveComment, self.comment))

        self.assertEqual(self.workflow.getInfoFor(self.forum, 'review_state'), 'memberposting')
        self.assertEqual(self.workflow.getInfoFor(self.conv, 'review_state'), 'active')
        self.assertEqual(self.workflow.getInfoFor(self.comment, 'review_state'), 'published')

# make_moderated disabled until moderation is fixed in general
#    def testAutoSubmitModerated(self):
#        self.workflow.doActionFor(self.forum, 'make_moderated')
#
#        self.login('member')
#
#        conv = self.forum.addConversation('conv2', 'conv2 body')
#        comment = conv.objectValues()[0]
#
#        self.failIf(checkPerm(permissions.ApproveComment, self.forum))
#        self.failIf(checkPerm(permissions.ApproveComment, self.conv))
#        self.failIf(checkPerm(permissions.ApproveComment, comment))
#
#        self.assertEqual(self.workflow.getInfoFor(self.forum, 'review_state'), 'moderated')
#        self.assertEqual(self.workflow.getInfoFor(conv, 'review_state'), 'pending')
#        self.assertEqual(self.workflow.getInfoFor(comment, 'review_state'), 'pending')

    def testCommentEditing(self):
        self.login('manager')

        conv = self.forum.addConversation('conv2', 'conv2 body')
        
        self.failUnless(checkPerm(permissions.EditComment, self.comment))
        
        self.logout()

        self.login('member2')
        self.failIf(checkPerm(permissions.EditComment, self.comment))


class TestWorkflowsCreation(PloneboardTestCase.PloneboardTestCase):

    def afterSetUp(self):
        self.workflow = self.portal.portal_workflow

    def testWorkflowsCreated(self):
        workflows = ['ploneboard_workflow', 'ploneboard_forum_workflow',
                     'ploneboard_conversation_workflow', 'ploneboard_comment_workflow']
        for workflow in workflows:
            self.failUnless(workflow in self.workflow.objectIds(), "%s missing" % workflow)

    def XXXtestPreserveChainsOnReinstall(self):
        # Disable this test: GenericSetup profiles will always overwrite the
        # workflow chains for the types
        boardtypes = ('Ploneboard',
                      'PloneboardForum',
                      'PloneboardConversation',
                      'PloneboardComment')

        self.workflow.setChainForPortalTypes(boardtypes, 'plone_workflow')
        self.workflow.getChainForPortalType('Ploneboard')
        for boardtype in boardtypes:
            self.failUnless('plone_workflow' in self.workflow.getChainForPortalType(boardtype),
                            'Workflow chain for %s not set' % boardtype)

        self.portal.portal_quickinstaller.reinstallProducts(['Ploneboard'])
        for boardtype in boardtypes:
            chain = self.workflow.getChainForPortalType(boardtype)
            self.failUnless('plone_workflow' in chain,
                            'Overwritten workflow chain for %s: %s' % (boardtype, ', '.join(chain)))

    def testPermissionsOnPortal(self):
        p=Permission('Ploneboard: Add Comment Attachment', (), self.portal)
        roles=p.getRoles()
        self.failUnless('Member' in roles)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCommentWorkflow))
    suite.addTest(makeSuite(TestWorkflowsCreation))
    return suite



"""
$Id: comment.py 55896 2007-12-19 14:36:54Z wichert $
"""
import urllib
from zope import interface
from Acquisition import aq_base
from DateTime.DateTime import DateTime
from Products import Five
from Products.CMFCore import utils as cmf_utils
from Products.Ploneboard import permissions
from Products.Ploneboard.batch import Batch
from Products.Ploneboard.browser.interfaces import IConversationView
from Products.Ploneboard.browser.interfaces import ICommentView
from Products.Ploneboard.utils import PloneboardMessageFactory as _

class CommentViewableView(Five.BrowserView):
    """Any view that might want to interact with comments should inherit
    from this base class.
    """

    def __init__(self, context, request):
        Five.BrowserView.__init__(self, context, request)

        self.portal_actions = cmf_utils.getToolByName(self.context, 'portal_actions')
        self.plone_utils = cmf_utils.getToolByName(self.context, 'plone_utils')
        self.portal_membership = cmf_utils.getToolByName(self.context, 'portal_membership')
        self.portal_workflow = cmf_utils.getToolByName(self.context, 'portal_workflow')

    def _buildDict(self, comment):
        """Produce a dict representative of all the important properties
        of a comment.
        """

        checkPermission = self.portal_membership.checkPermission
        actions = self.portal_actions.listFilteredActionsFor(comment)

        res= {
                'Title': comment.title_or_id(),
                'Creator': comment.Creator(),
                'creation_date': comment.CreationDate(),
                'getId': comment.getId(),
                'getText': comment.getText(),
                'absolute_url': comment.absolute_url(),
                'getAttachments': comment.getAttachments(),
                'canEdit': checkPermission(permissions.EditComment, comment),
                'canDelete': checkPermission(permissions.DeleteComment, comment),
                'canReply': checkPermission(permissions.AddComment, comment),
                'getObject': comment,
                'workflowActions' : actions['workflow'],
                'review_state' : self.portal_workflow.getInfoFor(comment, 'review_state'),
                'reviewStateTitle' : self.plone_utils.getReviewStateTitleFor(comment),
                'UID': comment.UID(),
            }
        return res

class CommentView(CommentViewableView):
    """A view for getting information about one specific comment.
    """

    interface.implements(ICommentView)

    def comment(self):
        return self._buildDict(self.context)

    def author(self):
        creator = self.context.Creator()
        info = self.portal_membership.getMemberInfo(creator)
        if info is None:
            return creator
        return info.get('fullname', creator)

    def quotedBody(self):
        text = self.context.getText()
        if text:
            return '<p>Previously %s wrote:</p>' \
                   '<blockquote>%s</blockquote><p></p>' % \
                   (self.author(), self.context.getText())
        else:
            return ''

class ConversationView(CommentView):
    """A view component for querying conversations.
    """

    interface.implements(IConversationView)

    def conversation(self):
        checkPermission = self.portal_membership.checkPermission
        conv = self.context
        forum = conv.getForum()

        return {
                'maximumAttachments' : forum.getMaxAttachments(),
                'maximumAttachmentSize' : forum.getMaxAttachmentSize(),
                'canAttach': forum.getMaxAttachments()>0 and \
                              checkPermission(permissions.AddAttachment,conv),
                }

    def comments(self):
        batchSize = 30
        batchStart = int(self.request.get('b_start', 0))
        numComments = self.context.getNumberOfComments()
        return Batch(self._getComments, numComments, batchSize, batchStart, orphan=1)

    def root_comments(self):
        rootcomments =  self.context.getRootComments()
        for ob in rootcomments:
            yield self._buildDict(ob)

    def children(self, comment):
        if type(comment) is dict:
            comment = comment['getObject']

        for ob in comment.getReplies():
            yield self._buildDict(ob)

    def _getComments(self, limit, offset):
        """Dictify comments before returning them to the batch
        """
        return [self._buildDict(ob) for ob in self.context.getComments(limit=limit, offset=offset)]


class RecentConversationsView(CommentViewableView):
    """Find recent conversations
    """

    def __init__(self, context, request):
        Five.BrowserView.__init__(self, context, request)
        self.portal_workflow = cmf_utils.getToolByName(self.context, 'portal_workflow')
        self.plone_utils = cmf_utils.getToolByName(self.context, 'plone_utils')
        self.portal_catalog = cmf_utils.getToolByName(self.context, 'portal_catalog')
        self.portal_membership = cmf_utils.getToolByName(self.context, 'portal_membership')

    def num_conversations(self):
        catalog = self.portal_catalog
        results = catalog(object_provides='Products.Ploneboard.interfaces.IConversation',
                          path='/'.join(self.context.getPhysicalPath()),)
        return len(results)

    def results(self, limit=20, offset=0):
        catalog = self.portal_catalog
        results = catalog(object_provides='Products.Ploneboard.interfaces.IConversation',
                            sort_on='modified',
                            sort_order='reverse',
                            sort_limit=(offset+limit),
                            path='/'.join(self.context.getPhysicalPath()))[offset:offset+limit]
        return filter(None, [self._buildDict(r.getObject()) for r in results])

    def _buildDict(self, ob):
        forum = ob.getForum()
        wfstate = self.portal_workflow.getInfoFor(ob, 'review_state')
        wfstate = self.plone_utils.normalizeString(wfstate)

        creator = ob.Creator()
        creatorInfo = self.portal_membership.getMemberInfo(creator)
        if creatorInfo is not None and creatorInfo.get('fullname', "") != "":
            creator = creatorInfo['fullname']

        lastComment = ob.getLastComment()
        if lastComment is None:
            return None
        canAccessLastComment = self.portal_membership.checkPermission('View', lastComment)

        lastCommentCreator = lastComment.Creator()
        creatorInfo = self.portal_membership.getMemberInfo(lastCommentCreator)
        if creatorInfo is not None and creatorInfo.get('fullname', '') != "":
            lastCommentCreator = creatorInfo['fullname']

        return { 'Title': ob.title_or_id(),
                 'Description' : ob.Description(),
                 'absolute_url': ob.absolute_url(),
                 'forum_title' : forum.title_or_id(),
                 'forum_url' : forum.absolute_url(),
                 'review_state_normalized' : wfstate,
                 'num_comments' : ob.getNumberOfComments(),
                 'creator' : creator,
                 'last_comment_id' : lastComment.getId(),
                 'last_comment_creator' : lastCommentCreator,
                 'last_comment_date' : lastComment.created(),
                 'can_access_last_comment' : canAccessLastComment,
                 'is_new' : self._is_new(ob.modified()),
               }

    def _is_new(self, modified):
        llt = getattr(aq_base(self), '_last_login_time', [])
        if llt == []:
            m = self.portal_membership.getAuthenticatedMember()
            if m.has_role('Anonymous'):
                llt = self._last_login_time = None
            else:
                llt = self._last_login_time = m.getProperty('last_login_time', 0)
        if llt is None: # not logged in
            return False
        elif llt == 0: # never logged in before
            return True
        else:
            return (modified >= DateTime(llt))

class UnansweredConversationsView(RecentConversationsView):
    """Find unanswered conversations
    """

    def num_conversations(self):
        catalog = self.portal_catalog
        results = catalog(object_provides='Products.Ploneboard.interfaces.IConversation',
                          num_comments=1,
                          path='/'.join(self.context.getPhysicalPath()),)
        return len(results)

    def results(self, limit=20, offset=0):
        catalog = self.portal_catalog
        results = catalog(object_provides='Products.Ploneboard.interfaces.IConversation',
                            num_comments=1,
                            sort_on='modified',
                            sort_order='reverse',
                            sort_limit=(offset+limit),
                            path='/'.join(self.context.getPhysicalPath()))[offset:offset+limit]
        return [self._buildDict(r.getObject()) for r in results]

    def _buildDict(self, ob):
        forum = ob.getForum()
        wfstate = self.portal_workflow.getInfoFor(ob, 'review_state')
        wfstate = self.plone_utils.normalizeString(wfstate)

        creator = ob.Creator()
        creatorInfo = self.portal_membership.getMemberInfo(creator)
        if creatorInfo is not None and creatorInfo.get('fullname', "") != "":
            creator = creatorInfo['fullname']

        return { 'Title': ob.title_or_id(),
                 'Description' : ob.Description(),
                 'created' : ob.created(),
                 'absolute_url': ob.absolute_url(),
                 'forum_title' : forum.title_or_id(),
                 'forum_url' : forum.absolute_url(),
                 'review_state_normalized' : wfstate,
                 'creator' : creator,
                 'is_new' : self._is_new(ob.modified()),
               }

class DeleteCommentView(Five.BrowserView):
    """Delete the current comment.  If the comment is the root comment
    of a conversation, delete the entire conversation instead.
    """

    def __call__(self):
        redirect = self.request.response.redirect
        comment = self.context
        conversation = comment.getConversation()
        plone_utils = cmf_utils.getToolByName(comment, 'plone_utils')

        if len(conversation.getComments()) == 1:
            forum = conversation.getForum()
            conversation.delete()
            msg = _(u'Conversation deleted')
            plone_utils.addPortalMessage(msg)
            redirect(forum.absolute_url())
        else:
            comment.delete()
            msg = _(u'Comment deleted')
            plone_utils.addPortalMessage(msg)
            redirect(conversation.absolute_url())




from setuptools import setup, find_packages

version = '2.0b2'

setup(name='Products.Ploneboard',
      version=version,
      description="A discussion board for Plone.",
      long_description=open("README.txt").read() + \
                        open("docs/INSTALL.txt").read() + \
                        open("docs/HISTORY.txt").read(),
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Programming Language :: Python",
      ],
      keywords='Zope CMF Plone board forum',
      author='Jarn, Wichert Akkerman, Martin Aspeli',
      author_email='plone-developers@lists.sourceforge.net',
      url='http://svn.plone.org/svn/collective/Ploneboard/trunk',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      download_url='http://plone.org/products/ploneboard',
      install_requires=[
        'setuptools',
        'Products.SimpleAttachment',
        'plone.app.portlets',
        'plone.portlets',
        'plone.memoize',
        'plone.i18n',
      ],
)



from ExtensionClass import Base
from Products.CMFPlone.PloneBatch import Batch as PloneBatch,        \
                                         calculate_pagenumber,       \
                                         calculate_pagerange,        \
                                         calculate_leapback,         \
                                         calculate_leapforward
                                         
class LazyPrevBatch(Base):
    def __of__(self, parent):
        start = parent.first - parent._size + parent.overlap
        if start < 0 or start >= parent.sequence_length:
            return None
        return Batch(parent._method, parent.sequence_length, parent._size,
                     start, 0, parent.orphan, parent.overlap)

class LazyNextBatch(Base):
    def __of__(self, parent):
        start = parent.end - parent.overlap
        if start < 0 or start >= parent.sequence_length:
            return None
        return Batch(parent._method, parent.sequence_length, parent._size,
                     start, 0, parent.orphan, parent.overlap)

class Batch(PloneBatch):
    """A custom batch implementation to enable batching over forums and
    conversations. This is identical to PloneBatch's implementation except:
    
     - The sequence length is explicitly given to the constructor
     - Instead of operating on a sequence, the batch is given a method to call.
       This method needs to have a signature
       
            def getItems(self, limit, offset)
       
       limit is the number of items to return (the batch size) and offset is
       the number of items to skip at the beginning of the sequence.
    """
    __allow_access_to_unprotected_subobjects__ = 1
    
    # Needs first, length, 
    
    previous = LazyPrevBatch()
    next = LazyNextBatch()
    sequence_length = None
    
    size = first= start = end = orphan = overlap = navlist = None
    numpages = pagenumber = pagerange = pagerangeend = pagerangestart = pagenumber = quantumleap = None
    
    
    def __init__( self, method, sequence_length, size, start=0, end=0, orphan=0, overlap=0, pagerange=7, quantumleap=0, b_start_str='b_start'):
        """ Encapsulate sequence in batches of size
        method          - the method to call to get the correct part of the batch
        sequence_length - the total number of items in the sequence
        size            - the number of items in each batch. This will be computed if left out.
        start           - the first element of sequence to include in batch (0-index)
        end             - the last element of sequence to include in batch (0-index, optional)
        orphan          - the next page will be combined with the current page if it does not contain more than orphan elements
        overlap         - the number of overlapping elements in each batch
        pagerange       - the number of pages to display in the navigation
        quantumleap     - 0 or 1 to indicate if bigger increments should be used in the navigation list for big results.
        b_start_str     - the request variable used for start, default 'b_start'
        """
        start = start + 1

        self.sequence_length = sequence_length
        self._method = method

        start,end,sz = opt(start,end,size,orphan,sequence_length)
        
        self.size = sz
        self._size = size
        self.start = start
        self.end = end
        self.orphan = orphan
        self.overlap = overlap
        self.first = max(start - 1, 0)
        self.length = self.end - self.first
        
        self.b_start_str = b_start_str

        self.last = sequence_length - size

        # Set up next and previous
        if self.first == 0:
            self.previous = None

        # Set up the total number of pages
        self.numpages = calculate_pagenumber(self.sequence_length - self.orphan, self.size, self.overlap)

        # Set up the current page number
        self.pagenumber = calculate_pagenumber(self.start, self.size, self.overlap)

        # Set up pagerange for the navigation quick links
        self.pagerange, self.pagerangestart, self.pagerangeend = calculate_pagerange(self.pagenumber,self.numpages,pagerange)

        # Set up the lists for the navigation: 4 5 [6] 7 8
        #  navlist is the complete list, including pagenumber
        #  prevlist is the 4 5 in the example above
        #  nextlist is 7 8 in the example above
        self.navlist = self.prevlist = self.nextlist = []
        if self.pagerange and self.numpages >= 1:
            self.navlist  = range(self.pagerangestart, self.pagerangeend)
            self.prevlist = range(self.pagerangestart, self.pagenumber)
            self.nextlist = range(self.pagenumber + 1, self.pagerangeend)

        # QuantumLeap - faster navigation for big result sets
        self.quantumleap = quantumleap
        self.leapback = self.leapforward = []
        if self.quantumleap:
            self.leapback = calculate_leapback(self.pagenumber, self.numpages, self.pagerange)
            self.leapforward = calculate_leapforward(self.pagenumber, self.numpages, self.pagerange)

    def __getitem__(self, index):
        """ Pull an item out of the partial sequence that is this batch """
        if (index < 0 and index + self.end < self.first) or index >= self.length:
            raise IndexError, index
        sequence = getattr(self, '_partial_sequence', None)
        if sequence is None:
            self._partial_sequence = sequence = self._method(self.length, self.start-1)
        return sequence[index]

    def __len__(self):
        """ Get the length of the partial sequence that is this batch """
        return self.length

def opt(start, end, size, orphan, sequence_length):
    """Calculate start, end and batch size"""
    length = sequence_length
    if size < 1:
        if start > 0 and end > 0 and end >= start:
            size = end + 1 - start
        else: size = 25
    if start > 0: 
        if start>length: 
            start = length
        if end > 0:
            if end < start: end = start
        else:
            end = start + size - 1
            if (end+orphan)>=length:
                end = length
    elif end > 0:
        if (end)>length:
            end = length
        start = end + 1 - size
        if start - 1 < orphan: start = 1
    else:
        start = 1
        end = start + size - 1
        if (end+orphan)>=length:
            end = length
    return start,end,size



from Products.CMFPlone.CatalogTool import registerIndexableAttribute
from Products.Ploneboard.interfaces import IConversation
from zope.interface import providedBy
from zope.component.interface import interfaceToName

try:
    from Products.CMFPlone.CatalogTool import object_provides
except ImportError:
    def object_provides(object, portal, **kw):
        return [interfaceToName(portal, i) for i in
                providedBy(object).flattened()]

    registerIndexableAttribute('object_provides', object_provides)


def num_comments(object, portal, **kw):
    conv = IConversation(object, None)
    if conv is None:
        return None
    else:
        return conv.getNumberOfComments()

registerIndexableAttribute('num_comments', num_comments)



from Products.CMFCore import permissions
from Products.CMFCore.permissions import setDefaultRoles

# Add permissions differ for each type, and are imported by __init__.initialize
# so don't change their names!

ViewBoard = permissions.View
SearchBoard = 'Ploneboard: Search'
AddBoard = AddPloneboard = 'Ploneboard: Add Ploneboard'
ManageBoard = 'Ploneboard: Add Ploneboard'

RequestReview = permissions.RequestReview

AddForum = AddPloneboardForum = 'Ploneboard: Add Forum'
ManageForum = 'Ploneboard: Add Forum'

AddConversation = AddPloneboardConversation = 'Ploneboard: Add Conversation'
ManageConversation = 'Ploneboard: Manage Conversation'

AddComment = AddPloneboardComment = 'Ploneboard: Add Comment'
EditComment = permissions.ModifyPortalContent
AddAttachment = AddPloneboardAttachment = 'Ploneboard: Add Comment Attachment'
ManageComment = 'Ploneboard: Manage Comment'
ApproveComment = 'Ploneboard: Approve Comment' # Used for moderation
RetractComment = 'Ploneboard: Retract Comment'
ModerateForum = 'Ploneboard: Moderate Forum'

# Note: if this changes, you must also change configure.zcml!
DeleteComment = permissions.DeleteObjects


# Set up default roles for permissions

setDefaultRoles(ViewBoard,
                ('Anonymous', 'Member', 'Manager'))

setDefaultRoles(SearchBoard,
                ('Anonymous', 'Member', 'Manager'))

setDefaultRoles(AddBoard,
                ('Manager', 'Owner'))

setDefaultRoles(ManageBoard,
                ('Manager', 'Owner'))

setDefaultRoles(AddConversation,
                ('Authenticated', 'Manager'))

setDefaultRoles(AddComment,
                ('Authenticated', 'Manager'))

setDefaultRoles(AddAttachment,
                ('Manager',))

setDefaultRoles(ManageConversation,
                ('Manager',))

setDefaultRoles(ManageComment,
                ('Manager',))

setDefaultRoles(ApproveComment,
                ('Manager',))

setDefaultRoles(RetractComment,
                ('Manager',))

setDefaultRoles(ModerateForum,
                ('Manager', 'Reviewer',))



## Script (Python) "moderateComment"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=action,cameFrom=None
##title=Moderate the given comment, and return to referer
##

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException

workflow = getToolByName(context, 'portal_workflow')
workflow.doActionFor(context, action)

if cameFrom is None:
    cameFrom = context.REQUEST.get('HTTP_REFERER', context.absolute_url())
context.REQUEST.RESPONSE.redirect(cameFrom)


from zope.interface import Interface

class IConversationView(Interface):
    def comments():
        """Return all comments in the conversation.
        """

    def conversation():
        """Return active conversation.
        """

    def root_comments():
        """Return all of the root comments for a conversation.
        """

    def children(comment):
        """Return all of the children comments for a parent comment.
        """

class ICommentView(Interface):

    def comment():
        """Return active comment.
        """

    def author():
        """Return the name of the author of this comment.
        If no full name is known the userid is returned.
        """

    def quotedBody():
        """Return the body of the comment, quoted for a reply.
        """


